@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2017, GrammarSoft ApS
+* Copyright (C) 2007-2018, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -27,21 +27,33 @@
 
 namespace CG3 {
 
-int BinaryGrammar::readBinaryGrammar(FILE *input) {
-	if (!input) {
-		u_fprintf(ux_stderr, "Error: Input is null - cannot read from nothing!\n");
-		CG3Quit(1);
-	}
-	if (!grammar) {
-		u_fprintf(ux_stderr, "Error: No grammar provided - cannot continue!\n");
-		CG3Quit(1);
-	}
+int BinaryGrammar::parse_grammar(const UChar*, size_t) {
+	throw "UChar* interface doesn't make sense for binary grammars.";
+}
+int BinaryGrammar::parse_grammar(UString&) {
+	throw "UString interface doesn't make sense for binary grammars.";
+}
+
+int BinaryGrammar::parse_grammar(const std::string& buffer) {
+	return parse_grammar(buffer.c_str(), buffer.size());
+}
+
+int BinaryGrammar::parse_grammar(const char* buffer, size_t length) {
+	std::stringstream input;
+	input.write(buffer, length);
+	input.seekg(0);
+	return parse_grammar(input);
+}
+
+int BinaryGrammar::parse_grammar(std::istream& input) {
+	input.exceptions(std::ios::failbit | std::ios::eofbit | std::ios::badbit);
+
 	uint32_t fields = 0;
 	uint32_t u32tmp = 0;
 	int32_t i32tmp = 0;
 	uint8_t u8tmp = 0;
 	UErrorCode err = U_ZERO_ERROR;
-	UConverter *conv = ucnv_open("UTF-8", &err);
+	UConverter* conv = ucnv_open("UTF-8", &err);
 	std::stringstream buffer;
 
 	if (fread_throw(&cbuffers[0][0], 1, 4, input) != 4) {
@@ -60,7 +72,7 @@ int BinaryGrammar::readBinaryGrammar(FILE *input) {
 			u_fprintf(ux_stderr, "Warning: Grammar revision is %u, but current format is %u or later. Please recompile the binary grammar with latest CG-3.\n", u32tmp, CG3_FEATURE_REV);
 			u_fflush(ux_stderr);
 		}
-		fseek(input, 0, SEEK_SET);
+		input.seekg(0);
 		return readBinaryGrammar_10043(input);
 	}
 	if (u32tmp < CG3_TOO_OLD) {
@@ -81,6 +93,7 @@ int BinaryGrammar::readBinaryGrammar(FILE *input) {
 	grammar->sub_readings_ltr = (fields & (1 << 2)) != 0;
 	grammar->has_relations = (fields & (1 << 13)) != 0;
 	grammar->has_bag_of_tags = (fields & (1 << 14)) != 0;
+	grammar->ordered = (fields & (1 << 15)) != 0;
 
 	if (fields & (1 << 1)) {
 		ucnv_reset(conv);
@@ -102,7 +115,7 @@ int BinaryGrammar::readBinaryGrammar(FILE *input) {
 	uint32_t num_single_tags = u32tmp;
 	grammar->single_tags_list.resize(num_single_tags);
 	for (uint32_t i = 0; i < num_single_tags; i++) {
-		Tag *t = grammar->allocateTag();
+		Tag* t = grammar->allocateTag();
 		t->type |= T_GRAMMAR;
 
 		uint32_t fields = 0;
@@ -149,7 +162,7 @@ int BinaryGrammar::readBinaryGrammar(FILE *input) {
 			}
 		}
 		if (fields & (1 << 12)) {
-			char buf[sizeof(uint64_t)+ sizeof(int32_t)] = {};
+			char buf[sizeof(uint64_t) + sizeof(int32_t)] = {};
 			fread_throw(&buf[0], sizeof(buf), 1, input);
 			buffer.str("");
 			buffer.clear();
@@ -290,7 +303,7 @@ int BinaryGrammar::readBinaryGrammar(FILE *input) {
 	uint32_t num_sets = u32tmp;
 	grammar->sets_list.resize(num_sets);
 	for (uint32_t i = 0; i < num_sets; i++) {
-		Set *s = grammar->allocateSet();
+		Set* s = grammar->allocateSet();
 
 		uint32_t fields = 0;
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
@@ -353,9 +366,9 @@ int BinaryGrammar::readBinaryGrammar(FILE *input) {
 
 	// Actually assign sets to the varstring tags now that sets are loaded
 	for (auto iter : tag_varsets) {
-		Tag *t = grammar->single_tags_list[iter.first];
+		Tag* t = grammar->single_tags_list[iter.first];
 		for (auto uit : iter.second) {
-			Set *s = grammar->sets_list[uit];
+			Set* s = grammar->sets_list[uit];
 			t->vs_sets->push_back(s);
 		}
 	}
@@ -379,7 +392,7 @@ int BinaryGrammar::readBinaryGrammar(FILE *input) {
 	}
 	uint32_t num_contexts = u32tmp;
 	for (uint32_t i = 0; i < num_contexts; i++) {
-		ContextualTest *t = readContextualTest(input);
+		ContextualTest* t = readContextualTest(input);
 		grammar->contexts[t->hash] = t;
 	}
 
@@ -391,7 +404,7 @@ int BinaryGrammar::readBinaryGrammar(FILE *input) {
 	uint32_t num_rules = u32tmp;
 	grammar->rule_by_number.resize(num_rules);
 	for (uint32_t i = 0; i < num_rules; i++) {
-		Rule *r = grammar->allocateRule();
+		Rule* r = grammar->allocateRule();
 
 		uint32_t fields = 0;
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
@@ -483,7 +496,7 @@ int BinaryGrammar::readBinaryGrammar(FILE *input) {
 		for (uint32_t j = 0; j < num_dep_tests; j++) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
 			u32tmp = (uint32_t)ntohl(u32tmp);
-			ContextualTest *t = grammar->contexts[u32tmp];
+			ContextualTest* t = grammar->contexts[u32tmp];
 			r->addContextualTest(t, r->dep_tests);
 		}
 
@@ -493,7 +506,7 @@ int BinaryGrammar::readBinaryGrammar(FILE *input) {
 		for (uint32_t j = 0; j < num_tests; j++) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
 			u32tmp = (uint32_t)ntohl(u32tmp);
-			ContextualTest *t = grammar->contexts[u32tmp];
+			ContextualTest* t = grammar->contexts[u32tmp];
 			r->addContextualTest(t, r->tests);
 		}
 		grammar->rule_by_number[r->number] = r;
@@ -516,8 +529,8 @@ int BinaryGrammar::readBinaryGrammar(FILE *input) {
 	return 0;
 }
 
-ContextualTest *BinaryGrammar::readContextualTest(FILE *input) {
-	ContextualTest *t = grammar->allocateContextualTest();
+ContextualTest* BinaryGrammar::readContextualTest(std::istream& input) {
+	ContextualTest* t = grammar->allocateContextualTest();
 	uint32_t fields = 0;
 	uint32_t u32tmp = 0;
 	int32_t i32tmp = 0;
