@@ -30,7 +30,7 @@ namespace CG3 {
 static std::vector<ContextualTest*> contexts_list;
 static Grammar::contexts_t templates;
 
-int BinaryGrammar::readBinaryGrammar_10043(FILE *input) {
+int BinaryGrammar::readBinaryGrammar_10043(std::istream& input) {
 	if (!input) {
 		u_fprintf(ux_stderr, "Error: Input is null - cannot read from nothing!\n");
 		CG3Quit(1);
@@ -44,7 +44,7 @@ int BinaryGrammar::readBinaryGrammar_10043(FILE *input) {
 	int32_t i32tmp = 0;
 	uint8_t u8tmp = 0;
 	UErrorCode err = U_ZERO_ERROR;
-	UConverter *conv = ucnv_open("UTF-8", &err);
+	UConverter* conv = ucnv_open("UTF-8", &err);
 
 	if (fread_throw(&cbuffers[0][0], 1, 4, input) != 4) {
 		std::cerr << "Error: Error reading first 4 bytes from grammar!" << std::endl;
@@ -91,7 +91,7 @@ int BinaryGrammar::readBinaryGrammar_10043(FILE *input) {
 	uint32_t num_single_tags = u32tmp;
 	grammar->single_tags_list.resize(num_single_tags);
 	for (uint32_t i = 0; i < num_single_tags; i++) {
-		Tag *t = grammar->allocateTag();
+		Tag* t = grammar->allocateTag();
 		t->type |= T_GRAMMAR;
 
 		uint32_t fields = 0;
@@ -258,7 +258,7 @@ int BinaryGrammar::readBinaryGrammar_10043(FILE *input) {
 	uint32_t num_sets = u32tmp;
 	grammar->sets_list.resize(num_sets);
 	for (uint32_t i = 0; i < num_sets; i++) {
-		Set *s = grammar->allocateSet();
+		Set* s = grammar->allocateSet();
 
 		uint32_t fields = 0;
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
@@ -325,9 +325,9 @@ int BinaryGrammar::readBinaryGrammar_10043(FILE *input) {
 
 	// Actually assign sets to the varstring tags now that sets are loaded
 	for (auto iter : tag_varsets) {
-		Tag *t = grammar->single_tags_list[iter.first];
+		Tag* t = grammar->single_tags_list[iter.first];
 		for (auto uit : iter.second) {
-			Set *s = grammar->sets_list[uit];
+			Set* s = grammar->sets_list[uit];
 			t->vs_sets->push_back(s);
 		}
 	}
@@ -352,7 +352,7 @@ int BinaryGrammar::readBinaryGrammar_10043(FILE *input) {
 	uint32_t num_contexts = u32tmp;
 	contexts_list.resize(num_contexts);
 	for (uint32_t i = 0; i < num_contexts; i++) {
-		ContextualTest *t = readContextualTest_10043(input);
+		ContextualTest* t = readContextualTest_10043(input);
 		grammar->contexts[t->hash] = t;
 		contexts_list[i] = t;
 	}
@@ -365,7 +365,7 @@ int BinaryGrammar::readBinaryGrammar_10043(FILE *input) {
 	uint32_t num_rules = u32tmp;
 	grammar->rule_by_number.resize(num_rules);
 	for (uint32_t i = 0; i < num_rules; i++) {
-		Rule *r = grammar->allocateRule();
+		Rule* r = grammar->allocateRule();
 
 		uint32_t fields = 0;
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
@@ -457,7 +457,7 @@ int BinaryGrammar::readBinaryGrammar_10043(FILE *input) {
 		for (uint32_t j = 0; j < num_dep_tests; j++) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
 			u32tmp = (uint32_t)ntohl(u32tmp);
-			ContextualTest *t = contexts_list[u32tmp - 1];
+			ContextualTest* t = contexts_list[u32tmp - 1];
 			r->addContextualTest(t, r->dep_tests);
 		}
 
@@ -467,7 +467,7 @@ int BinaryGrammar::readBinaryGrammar_10043(FILE *input) {
 		for (uint32_t j = 0; j < num_tests; j++) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
 			u32tmp = (uint32_t)ntohl(u32tmp);
-			ContextualTest *t = contexts_list[u32tmp - 1];
+			ContextualTest* t = contexts_list[u32tmp - 1];
 			r->addContextualTest(t, r->tests);
 		}
 		grammar->rule_by_number[r->number] = r;
@@ -486,8 +486,8 @@ int BinaryGrammar::readBinaryGrammar_10043(FILE *input) {
 	return 0;
 }
 
-ContextualTest *BinaryGrammar::readContextualTest_10043(FILE *input) {
-	ContextualTest *t = grammar->allocateContextualTest();
+ContextualTest* BinaryGrammar::readContextualTest_10043(std::istream& input) {
+	ContextualTest* t = grammar->allocateContextualTest();
 	uint32_t fields = 0;
 	uint32_t u32tmp = 0;
 	int32_t i32tmp = 0;
@@ -551,7 +551,7 @@ ContextualTest *BinaryGrammar::readContextualTest_10043(FILE *input) {
 		for (uint32_t i = 0; i < num_ors; ++i) {
 			fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
 			u32tmp = (uint32_t)ntohl(u32tmp);
-			ContextualTest *to = contexts_list[u32tmp - 1];
+			ContextualTest* to = contexts_list[u32tmp - 1];
 			t->ors.push_back(to);
 		}
 	}
