@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2017, GrammarSoft ApS
+* Copyright (C) 2007-2018, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -27,7 +27,7 @@
 
 namespace CG3 {
 
-int BinaryGrammar::writeBinaryGrammar(FILE *output) {
+int BinaryGrammar::writeBinaryGrammar(FILE* output) {
 	if (!output) {
 		u_fprintf(ux_stderr, "Error: Output is null - cannot write to nothing!\n");
 		CG3Quit(1);
@@ -40,7 +40,7 @@ int BinaryGrammar::writeBinaryGrammar(FILE *output) {
 	uint32_t u32tmp = 0;
 	int32_t i32tmp = 0;
 	UErrorCode err = U_ZERO_ERROR;
-	UConverter *conv = ucnv_open("UTF-8", &err);
+	UConverter* conv = ucnv_open("UTF-8", &err);
 	std::ostringstream buffer;
 
 	fprintf(output, "CG3B");
@@ -94,6 +94,9 @@ int BinaryGrammar::writeBinaryGrammar(FILE *output) {
 	if (grammar->has_bag_of_tags) {
 		fields |= (1 << 14);
 	}
+	if (grammar->ordered) {
+		fields |= (1 << 15);
+	}
 
 	u32tmp = (uint32_t)htonl((uint32_t)fields);
 	fwrite_throw(&u32tmp, sizeof(uint32_t), 1, output);
@@ -110,10 +113,7 @@ int BinaryGrammar::writeBinaryGrammar(FILE *output) {
 		u32tmp = (uint32_t)htonl((uint32_t)grammar->single_tags_list.size());
 		fwrite_throw(&u32tmp, sizeof(uint32_t), 1, output);
 	}
-	std::vector<Tag*>::const_iterator tags_iter;
-	for (tags_iter = grammar->single_tags_list.begin(); tags_iter != grammar->single_tags_list.end(); tags_iter++) {
-		const Tag *t = *tags_iter;
-
+	for (auto t : grammar->single_tags_list) {
 		uint32_t fields = 0;
 		buffer.str("");
 		buffer.clear();
@@ -156,7 +156,7 @@ int BinaryGrammar::writeBinaryGrammar(FILE *output) {
 		if (!t->tag.empty()) {
 			fields |= (1 << 8);
 			ucnv_reset(conv);
-			i32tmp = ucnv_fromUChars(conv, &cbuffers[0][0], CG3_BUFFER_SIZE - 1, t->tag.c_str(), t->tag.size(), &err);
+			i32tmp = ucnv_fromUChars(conv, &cbuffers[0][0], CG3_BUFFER_SIZE - 1, t->tag.c_str(), static_cast<int32_t>(t->tag.size()), &err);
 			writeSwapped(buffer, i32tmp);
 			buffer.write(&cbuffers[0][0], i32tmp);
 		}
@@ -164,7 +164,7 @@ int BinaryGrammar::writeBinaryGrammar(FILE *output) {
 		if (t->regexp) {
 			fields |= (1 << 9);
 			int32_t len = 0;
-			const UChar *p = uregex_pattern(t->regexp, &len, &err);
+			const UChar* p = uregex_pattern(t->regexp, &len, &err);
 			ucnv_reset(conv);
 			i32tmp = ucnv_fromUChars(conv, &cbuffers[0][0], CG3_BUFFER_SIZE - 1, p, len, &err);
 			writeSwapped(buffer, i32tmp);
@@ -173,17 +173,17 @@ int BinaryGrammar::writeBinaryGrammar(FILE *output) {
 
 		if (t->vs_sets) {
 			fields |= (1 << 10);
-			writeSwapped<uint32_t>(buffer, t->vs_sets->size());
+			writeSwapped<uint32_t>(buffer, static_cast<uint32_t>(t->vs_sets->size()));
 			for (auto iter : *t->vs_sets) {
 				writeSwapped(buffer, iter->number);
 			}
 		}
 		if (t->vs_names) {
 			fields |= (1 << 11);
-			writeSwapped<uint32_t>(buffer, t->vs_names->size());
+			writeSwapped<uint32_t>(buffer, static_cast<uint32_t>(t->vs_names->size()));
 			for (auto iter : *t->vs_names) {
 				ucnv_reset(conv);
-				i32tmp = ucnv_fromUChars(conv, &cbuffers[0][0], CG3_BUFFER_SIZE - 1, iter.c_str(), iter.size(), &err);
+				i32tmp = ucnv_fromUChars(conv, &cbuffers[0][0], CG3_BUFFER_SIZE - 1, iter.c_str(), static_cast<int32_t>(iter.size()), &err);
 				writeSwapped(buffer, i32tmp);
 				buffer.write(&cbuffers[0][0], i32tmp);
 			}
@@ -192,15 +192,16 @@ int BinaryGrammar::writeBinaryGrammar(FILE *output) {
 
 		u32tmp = (uint32_t)htonl(fields);
 		fwrite_throw(&u32tmp, sizeof(uint32_t), 1, output);
-		fwrite_throw(buffer.str().c_str(), buffer.str().length(), 1, output);
+		const auto& str = buffer.str();
+		fwrite_throw(str.c_str(), str.size(), 1, output);
 	}
 
 	if (!grammar->reopen_mappings.empty()) {
 		u32tmp = (uint32_t)htonl((uint32_t)grammar->reopen_mappings.size());
 		fwrite_throw(&u32tmp, sizeof(uint32_t), 1, output);
 	}
-	for (auto iter = grammar->reopen_mappings.begin(); iter != grammar->reopen_mappings.end(); ++iter) {
-		u32tmp = (uint32_t)htonl((uint32_t)*iter);
+	for (auto iter : grammar->reopen_mappings) {
+		u32tmp = (uint32_t)htonl((uint32_t)iter);
 		fwrite_throw(&u32tmp, sizeof(uint32_t), 1, output);
 	}
 
@@ -208,8 +209,8 @@ int BinaryGrammar::writeBinaryGrammar(FILE *output) {
 		u32tmp = (uint32_t)htonl((uint32_t)grammar->preferred_targets.size());
 		fwrite_throw(&u32tmp, sizeof(uint32_t), 1, output);
 	}
-	for (auto iter = grammar->preferred_targets.begin(); iter != grammar->preferred_targets.end(); ++iter) {
-		u32tmp = (uint32_t)htonl((uint32_t)*iter);
+	for (auto iter : grammar->preferred_targets) {
+		u32tmp = (uint32_t)htonl((uint32_t)iter);
 		fwrite_throw(&u32tmp, sizeof(uint32_t), 1, output);
 	}
 
@@ -239,10 +240,7 @@ int BinaryGrammar::writeBinaryGrammar(FILE *output) {
 		u32tmp = (uint32_t)htonl((uint32_t)grammar->sets_list.size());
 		fwrite_throw(&u32tmp, sizeof(uint32_t), 1, output);
 	}
-	std::vector<Set*>::const_iterator set_iter;
-	for (set_iter = grammar->sets_list.begin(); set_iter != grammar->sets_list.end(); set_iter++) {
-		Set *s = *set_iter;
-
+	for (auto s : grammar->sets_list) {
 		uint32_t fields = 0;
 		buffer.str("");
 		buffer.clear();
@@ -258,21 +256,21 @@ int BinaryGrammar::writeBinaryGrammar(FILE *output) {
 		}
 		if (!s->getNonEmpty().empty()) {
 			fields |= (1 << 3);
-			writeSwapped<uint32_t>(buffer, s->trie.size());
+			writeSwapped<uint32_t>(buffer, static_cast<uint32_t>(s->trie.size()));
 			trie_serialize(s->trie, buffer);
-			writeSwapped<uint32_t>(buffer, s->trie_special.size());
+			writeSwapped<uint32_t>(buffer, static_cast<uint32_t>(s->trie_special.size()));
 			trie_serialize(s->trie_special, buffer);
 		}
 		if (!s->set_ops.empty()) {
 			fields |= (1 << 4);
-			writeSwapped<uint32_t>(buffer, s->set_ops.size());
+			writeSwapped<uint32_t>(buffer, static_cast<uint32_t>(s->set_ops.size()));
 			for (auto iter : s->set_ops) {
 				writeSwapped(buffer, iter);
 			}
 		}
 		if (!s->sets.empty()) {
 			fields |= (1 << 5);
-			writeSwapped<uint32_t>(buffer, s->sets.size());
+			writeSwapped<uint32_t>(buffer, static_cast<uint32_t>(s->sets.size()));
 			for (auto iter : s->sets) {
 				writeSwapped(buffer, iter);
 			}
@@ -280,14 +278,15 @@ int BinaryGrammar::writeBinaryGrammar(FILE *output) {
 		if (s->type & ST_STATIC) {
 			fields |= (1 << 6);
 			ucnv_reset(conv);
-			i32tmp = ucnv_fromUChars(conv, &cbuffers[0][0], CG3_BUFFER_SIZE - 1, s->name.c_str(), s->name.length(), &err);
+			i32tmp = ucnv_fromUChars(conv, &cbuffers[0][0], CG3_BUFFER_SIZE - 1, s->name.c_str(), static_cast<int32_t>(s->name.size()), &err);
 			writeSwapped(buffer, i32tmp);
 			buffer.write(&cbuffers[0][0], i32tmp);
 		}
 
 		u32tmp = (uint32_t)htonl(fields);
 		fwrite_throw(&u32tmp, sizeof(uint32_t), 1, output);
-		fwrite_throw(buffer.str().c_str(), buffer.str().length(), 1, output);
+		const auto& str = buffer.str();
+		fwrite_throw(str.c_str(), str.size(), 1, output);
 	}
 
 	if (grammar->delimiters) {
@@ -305,8 +304,8 @@ int BinaryGrammar::writeBinaryGrammar(FILE *output) {
 		u32tmp = (uint32_t)htonl((uint32_t)grammar->contexts.size());
 		fwrite_throw(&u32tmp, sizeof(uint32_t), 1, output);
 	}
-	for (auto cntx = grammar->contexts.begin(); cntx != grammar->contexts.end(); ++cntx) {
-		writeContextualTest(cntx->second, output);
+	for (auto& cntx : grammar->contexts) {
+		writeContextualTest(cntx.second, output);
 	}
 
 	if (!grammar->rule_by_number.empty()) {
@@ -337,7 +336,7 @@ int BinaryGrammar::writeBinaryGrammar(FILE *output) {
 		if (!r->name.empty()) {
 			fields |= (1 << 4);
 			ucnv_reset(conv);
-			i32tmp = ucnv_fromUChars(conv, &cbuffers[0][0], CG3_BUFFER_SIZE - 1, r->name.c_str(), r->name.size(), &err);
+			i32tmp = ucnv_fromUChars(conv, &cbuffers[0][0], CG3_BUFFER_SIZE - 1, r->name.c_str(), static_cast<int32_t>(r->name.size()), &err);
 			writeSwapped(buffer, i32tmp);
 			buffer.write(&cbuffers[0][0], i32tmp);
 		}
@@ -388,7 +387,8 @@ int BinaryGrammar::writeBinaryGrammar(FILE *output) {
 
 		u32tmp = (uint32_t)htonl(fields);
 		fwrite_throw(&u32tmp, sizeof(uint32_t), 1, output);
-		fwrite_throw(buffer.str().c_str(), buffer.str().length(), 1, output);
+		const auto& str = buffer.str();
+		fwrite_throw(str.c_str(), str.size(), 1, output);
 
 		u32tmp = 0;
 		if (r->dep_target) {
@@ -397,14 +397,14 @@ int BinaryGrammar::writeBinaryGrammar(FILE *output) {
 		fwrite_throw(&u32tmp, sizeof(uint32_t), 1, output);
 
 		r->reverseContextualTests();
-		u32tmp = (uint32_t)htonl(r->dep_tests.size());
+		u32tmp = (uint32_t)htonl(static_cast<unsigned long>(r->dep_tests.size()));
 		fwrite_throw(&u32tmp, sizeof(uint32_t), 1, output);
 		for (auto it : r->dep_tests) {
 			u32tmp = (uint32_t)htonl(it->hash);
 			fwrite_throw(&u32tmp, sizeof(uint32_t), 1, output);
 		}
 
-		u32tmp = (uint32_t)htonl(r->tests.size());
+		u32tmp = (uint32_t)htonl(static_cast<unsigned long>(r->tests.size()));
 		fwrite_throw(&u32tmp, sizeof(uint32_t), 1, output);
 		for (auto it : r->tests) {
 			u32tmp = (uint32_t)htonl(it->hash);
@@ -416,7 +416,7 @@ int BinaryGrammar::writeBinaryGrammar(FILE *output) {
 	return 0;
 }
 
-void BinaryGrammar::writeContextualTest(ContextualTest *t, FILE *output) {
+void BinaryGrammar::writeContextualTest(ContextualTest* t, FILE* output) {
 	if (seen_uint32.count(t->hash)) {
 		return;
 	}
@@ -492,7 +492,8 @@ void BinaryGrammar::writeContextualTest(ContextualTest *t, FILE *output) {
 
 	u32tmp = (uint32_t)htonl(fields);
 	fwrite_throw(&u32tmp, sizeof(uint32_t), 1, output);
-	fwrite_throw(buffer.str().c_str(), buffer.str().length(), 1, output);
+	const auto& str = buffer.str();
+	fwrite_throw(str.c_str(), str.size(), 1, output);
 
 	if (!t->ors.empty()) {
 		u32tmp = (uint32_t)htonl((uint32_t)t->ors.size());
