@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2017, GrammarSoft ApS
+* Copyright (C) 2007-2018, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -33,8 +33,8 @@ namespace CG3 {
 CohortVector pool_cohorts;
 pool_cleaner<CohortVector> cleaner_cohorts(pool_cohorts);
 
-Cohort *alloc_cohort(SingleWindow *p) {
-	Cohort *c = pool_get(pool_cohorts);
+Cohort* alloc_cohort(SingleWindow* p) {
+	Cohort* c = pool_get(pool_cohorts);
 	if (c == 0) {
 		c = new Cohort(p);
 	}
@@ -44,14 +44,14 @@ Cohort *alloc_cohort(SingleWindow *p) {
 	return c;
 }
 
-void free_cohort(Cohort *c) {
+void free_cohort(Cohort* c) {
 	if (c == 0) {
 		return;
 	}
 	pool_put(pool_cohorts, c);
 }
 
-Cohort::Cohort(SingleWindow *p)
+Cohort::Cohort(SingleWindow* p)
   : type(0)
   , global_number(0)
   , local_number(0)
@@ -162,19 +162,29 @@ void Cohort::remChild(uint32_t child) {
 	dep_children.erase(child);
 }
 
-void Cohort::appendReading(Reading *read) {
+void Cohort::appendReading(Reading* read) {
 	readings.push_back(read);
 	if (read->number == 0) {
-		read->number = (uint32_t)readings.size() * 1000 + 1000;
+		read->number = static_cast<uint32_t>(readings.size() * 1000 + 1000);
 	}
 	type &= ~CT_NUM_CURRENT;
 }
 
-Reading *Cohort::allocateAppendReading() {
-	Reading *read = alloc_reading(this);
+Reading* Cohort::allocateAppendReading() {
+	Reading* read = alloc_reading(this);
 	readings.push_back(read);
 	if (read->number == 0) {
-		read->number = (uint32_t)readings.size() * 1000 + 1000;
+		read->number = static_cast<uint32_t>(readings.size() * 1000 + 1000);
+	}
+	type &= ~CT_NUM_CURRENT;
+	return read;
+}
+
+Reading* Cohort::allocateAppendReading(Reading& r) {
+	Reading* read = alloc_reading(r);
+	readings.push_back(read);
+	if (read->number == 0) {
+		read->number = static_cast<uint32_t>(readings.size() * 1000 + 1000);
 	}
 	type &= ~CT_NUM_CURRENT;
 	return read;
@@ -188,7 +198,7 @@ void Cohort::updateMinMax() {
 	num_max.clear();
 	for (auto rter : readings) {
 		for (auto nter : rter->tags_numerical) {
-			const Tag *tag = nter.second;
+			const Tag* tag = nter.second;
 			if (num_min.find(tag->comparison_hash) == num_min.end() || tag->comparison_val < num_min[tag->comparison_hash]) {
 				num_min[tag->comparison_hash] = tag->comparison_val;
 			}
