@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2017, GrammarSoft ApS
+* Copyright (C) 2007-2018, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -32,7 +32,7 @@ class ContextualTest;
 
 class CohortIterator : public std::iterator<std::input_iterator_tag, Cohort*> {
 public:
-	CohortIterator(Cohort *cohort = 0, const ContextualTest *test = 0, bool span = false);
+	CohortIterator(Cohort* cohort = 0, const ContextualTest* test = 0, bool span = false);
 
 	virtual ~CohortIterator();
 
@@ -41,37 +41,37 @@ public:
 
 	virtual CohortIterator& operator++();
 
-	Cohort *operator*();
+	Cohort* operator*();
 
-	virtual void reset(Cohort *cohort = 0, const ContextualTest *test = 0, bool span = false);
+	virtual void reset(Cohort* cohort = 0, const ContextualTest* test = 0, bool span = false);
 
 protected:
 	bool m_span;
-	Cohort *m_cohort;
-	const ContextualTest *m_test;
+	Cohort* m_cohort;
+	const ContextualTest* m_test;
 };
 
 class TopologyLeftIter : public CohortIterator {
 public:
-	TopologyLeftIter(Cohort *cohort = 0, const ContextualTest *test = 0, bool span = false);
+	TopologyLeftIter(Cohort* cohort = 0, const ContextualTest* test = 0, bool span = false);
 
 	TopologyLeftIter& operator++();
 };
 
 class TopologyRightIter : public CohortIterator {
 public:
-	TopologyRightIter(Cohort *cohort = 0, const ContextualTest *test = 0, bool span = false);
+	TopologyRightIter(Cohort* cohort = 0, const ContextualTest* test = 0, bool span = false);
 
 	TopologyRightIter& operator++();
 };
 
 class DepParentIter : public CohortIterator {
 public:
-	DepParentIter(Cohort *cohort = 0, const ContextualTest *test = 0, bool span = false);
+	DepParentIter(Cohort* cohort = 0, const ContextualTest* test = 0, bool span = false);
 
 	DepParentIter& operator++();
 
-	void reset(Cohort *cohort = 0, const ContextualTest *test = 0, bool span = false);
+	void reset(Cohort* cohort = 0, const ContextualTest* test = 0, bool span = false);
 
 protected:
 	CohortSet m_seen;
@@ -79,11 +79,11 @@ protected:
 
 class DepDescendentIter : public CohortIterator {
 public:
-	DepDescendentIter(Cohort *cohort = 0, const ContextualTest *test = 0, bool span = false);
+	DepDescendentIter(Cohort* cohort = 0, const ContextualTest* test = 0, bool span = false);
 
 	DepDescendentIter& operator++();
 
-	void reset(Cohort *cohort = 0, const ContextualTest *test = 0, bool span = false);
+	void reset(Cohort* cohort = 0, const ContextualTest* test = 0, bool span = false);
 
 protected:
 	CohortSet m_descendents;
@@ -92,11 +92,11 @@ protected:
 
 class DepAncestorIter : public CohortIterator {
 public:
-	DepAncestorIter(Cohort *cohort = 0, const ContextualTest *test = 0, bool span = false);
+	DepAncestorIter(Cohort* cohort = 0, const ContextualTest* test = 0, bool span = false);
 
 	DepAncestorIter& operator++();
 
-	void reset(Cohort *cohort = 0, const ContextualTest *test = 0, bool span = false);
+	void reset(Cohort* cohort = 0, const ContextualTest* test = 0, bool span = false);
 
 protected:
 	CohortSet m_ancestors;
@@ -105,21 +105,21 @@ protected:
 
 class CohortSetIter : public CohortIterator {
 public:
-	CohortSetIter(Cohort *cohort = 0, const ContextualTest *test = 0, bool span = false);
+	CohortSetIter(Cohort* cohort = 0, const ContextualTest* test = 0, bool span = false);
 
-	void addCohort(Cohort *cohort);
+	void addCohort(Cohort* cohort);
 
 	CohortSetIter& operator++();
 
 protected:
-	Cohort *m_origcohort;
+	Cohort* m_origcohort;
 	CohortSet m_cohortset;
 	CohortSet::const_iterator m_cohortsetiter;
 };
 
 class MultiCohortIterator : public std::iterator<std::input_iterator_tag, Cohort*> {
 public:
-	MultiCohortIterator(Cohort *cohort = 0, const ContextualTest *test = 0, bool span = false);
+	MultiCohortIterator(Cohort* cohort = 0, const ContextualTest* test = 0, bool span = false);
 
 	virtual ~MultiCohortIterator();
 
@@ -128,19 +128,19 @@ public:
 
 	virtual MultiCohortIterator& operator++();
 
-	CohortIterator *operator*();
+	CohortIterator* operator*();
 
 protected:
 	bool m_span;
-	Cohort *m_cohort;
-	const ContextualTest *m_test;
+	Cohort* m_cohort;
+	const ContextualTest* m_test;
 	CohortSet m_seen;
-	CohortSetIter *m_cohortiter;
+	std::unique_ptr<CohortSetIter> m_cohortiter;
 };
 
 class ChildrenIterator : public MultiCohortIterator {
 public:
-	ChildrenIterator(Cohort *cohort = 0, const ContextualTest *test = 0, bool span = false);
+	ChildrenIterator(Cohort* cohort = 0, const ContextualTest* test = 0, bool span = false);
 
 	ChildrenIterator& operator++();
 
