@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2017, GrammarSoft ApS
+* Copyright (C) 2007-2018, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -32,6 +32,7 @@ Grammar::Grammar()
   , has_bag_of_tags(false)
   , has_relations(false)
   , has_encl_final(false)
+  , has_protect(false)
   , is_binary(false)
   , sub_readings_ltr(false)
   , grammar_size(0)
@@ -57,11 +58,8 @@ Grammar::~Grammar() {
 		delete rsets;
 	}
 
-	Taguint32HashMap::iterator iter_stag;
-	for (iter_stag = single_tags.begin(); iter_stag != single_tags.end(); ++iter_stag) {
-		if (iter_stag->second) {
-			delete iter_stag->second;
-		}
+	for (auto& iter_stag : single_tags) {
+		delete iter_stag.second;
 	}
 
 	for (auto iter_rules : rule_by_number) {
@@ -73,7 +71,7 @@ Grammar::~Grammar() {
 	}
 }
 
-void Grammar::addSet(Set *& to) {
+void Grammar::addSet(Set*& to) {
 	if (!delimiters && u_strcmp(to->name.c_str(), stringbits[S_DELIMITSET].getTerminatedBuffer()) == 0) {
 		delimiters = to;
 	}
@@ -91,7 +89,7 @@ void Grammar::addSet(Set *& to) {
 				all_tags = false;
 				break;
 			}
-			Set *s = getSet(to->sets[i]);
+			Set* s = getSet(to->sets[i]);
 			if (!s->sets.empty()) {
 				all_tags = false;
 				break;
@@ -107,8 +105,8 @@ void Grammar::addSet(Set *& to) {
 		}
 
 		if (all_tags) {
-			for (size_t i = 0; i < to->sets.size(); ++i) {
-				Set *s = getSet(to->sets[i]);
+			for (auto i : to->sets) {
+				Set* s = getSet(i);
 				maybe_used_sets.insert(s);
 				TagVector tv = trie_getTagList(s->getNonEmpty());
 				if (tv.size() == 1) {
@@ -143,8 +141,8 @@ void Grammar::addSet(Set *& to) {
 
 	// If there are failfast tags, and if they don't comprise the whole of the set, split the set into Positive - Negative
 	if (!to->ff_tags.empty() && to->ff_tags.size() < (to->trie.size() + to->trie_special.size())) {
-		Set *positive = allocateSet();
-		Set *negative = allocateSet();
+		Set* positive = allocateSet();
+		Set* negative = allocateSet();
 
 		UString str;
 		str = stringbits[S_GPREFIX].getTerminatedBuffer();
@@ -171,7 +169,7 @@ void Grammar::addSet(Set *& to) {
 					positive->trie_special.erase(pit);
 				}
 			}
-			Tag *tag = new Tag(*iter);
+			Tag* tag = new Tag(*iter);
 			tag->type &= ~T_FAILFAST;
 			tag = addTag(tag);
 			addTagToSet(tag, negative);
@@ -199,7 +197,7 @@ void Grammar::addSet(Set *& to) {
 	for (; to->name[0] != '_' || to->name[1] != 'G' || to->name[2] != '_';) {
 		uint32_t nhash = hash_value(to->name.c_str());
 		if (sets_by_name.find(nhash) != sets_by_name.end()) {
-			Set *a = sets_by_contents.find(sets_by_name.find(nhash)->second)->second;
+			Set* a = sets_by_contents.find(sets_by_name.find(nhash)->second)->second;
 			if (a == to || a->hash == to->hash) {
 				break;
 			}
@@ -212,7 +210,7 @@ void Grammar::addSet(Set *& to) {
 			break;
 		}
 		else if (chash != sets_by_contents.find(sets_by_name.find(nhash)->second)->second->hash) {
-			Set *a = sets_by_contents.find(sets_by_name.find(nhash)->second)->second;
+			Set* a = sets_by_contents.find(sets_by_name.find(nhash)->second)->second;
 			if (a->name == to->name) {
 				u_fprintf(ux_stderr, "Error: Set %S already defined at line %u. Redefinition attempted at line %u!\n", a->name.c_str(), a->line, to->line);
 				CG3Quit(1);
@@ -237,7 +235,7 @@ void Grammar::addSet(Set *& to) {
 		sets_by_contents[chash] = to;
 	}
 	else {
-		Set *a = sets_by_contents.find(chash)->second;
+		Set* a = sets_by_contents.find(chash)->second;
 		if (a != to) {
 			a->reindex(*this);
 			to->reindex(*this);
@@ -251,7 +249,7 @@ void Grammar::addSet(Set *& to) {
 	to = sets_by_contents[chash];
 }
 
-Set *Grammar::getSet(uint32_t which) const {
+Set* Grammar::getSet(uint32_t which) const {
 	Setuint32HashMap::const_iterator iter = sets_by_contents.find(which);
 	if (iter != sets_by_contents.end()) {
 		return iter->second;
@@ -274,18 +272,18 @@ Set *Grammar::getSet(uint32_t which) const {
 	return 0;
 }
 
-Set *Grammar::allocateSet() {
-	Set *ns = new Set;
+Set* Grammar::allocateSet() {
+	Set* ns = new Set;
 	sets_all.insert(ns);
 	return ns;
 }
 
-void Grammar::destroySet(Set *set) {
+void Grammar::destroySet(Set* set) {
 	sets_all.erase(set);
 	delete set;
 }
 
-void Grammar::addSetToList(Set *s) {
+void Grammar::addSetToList(Set* s) {
 	if (s->number == 0) {
 		if (sets_list.empty() || sets_list[0] != s) {
 			if (!s->sets.empty()) {
@@ -294,16 +292,16 @@ void Grammar::addSetToList(Set *s) {
 				}
 			}
 			sets_list.push_back(s);
-			s->number = (uint32_t)sets_list.size() - 1;
+			s->number = static_cast<uint32_t>(sets_list.size() - 1);
 		}
 	}
 }
 
 void Grammar::allocateDummySet() {
-	Set *set_c = allocateSet();
+	Set* set_c = allocateSet();
 	set_c->line = 0;
 	set_c->setName(stringbits[S_IGNORE].getTerminatedBuffer());
-	Tag *t = allocateTag(stringbits[S_IGNORE].getTerminatedBuffer());
+	Tag* t = allocateTag(stringbits[S_IGNORE].getTerminatedBuffer());
 	addTagToSet(t, set_c);
 	addSet(set_c);
 	set_c->number = std::numeric_limits<uint32_t>::max();
@@ -311,24 +309,24 @@ void Grammar::allocateDummySet() {
 }
 
 uint32_t Grammar::removeNumericTags(uint32_t s) {
-	Set *set = getSet(s);
+	Set* set = getSet(s);
 	if (!set->sets.empty()) {
 		bool did = false;
 		auto sets = set->sets;
-		for (size_t i = 0; i < sets.size(); ++i) {
-			uint32_t ns = removeNumericTags(sets[i]);
+		for (auto& i : sets) {
+			uint32_t ns = removeNumericTags(i);
 			if (ns == 0) {
-				set = getSet(sets[i]);
+				set = getSet(i);
 				u_fprintf(ux_stderr, "Error: Removing numeric tags for branch resulted in set %S on line %u being empty!\n", set->name.c_str(), set->line);
 				CG3Quit(1);
 			}
-			if (ns != sets[i]) {
-				sets[i] = ns;
+			if (ns != i) {
+				i = ns;
 				did = true;
 			}
 		}
 		if (did) {
-			Set *ns = allocateSet();
+			Set* ns = allocateSet();
 			ns->type = set->type;
 			ns->line = set->line;
 			ns->name = stringbits[S_GPREFIX].getTerminatedBuffer();
@@ -346,16 +344,16 @@ uint32_t Grammar::removeNumericTags(uint32_t s) {
 		bool did = false;
 		std::map<TagVector, bool> ntags;
 		TagVector tags;
-		const trie_t *tries[2] = { &set->trie, &set->trie_special };
+		const trie_t* tries[2] = { &set->trie, &set->trie_special };
 		for (size_t i = 0; i < 2; ++i) {
 			if (tries[i]->empty()) {
 				continue;
 			}
 			auto ctags = trie_getTags(*tries[i]);
-			for (auto it = ctags.begin(); it != ctags.end(); ++it) {
+			for (auto& it : ctags) {
 				bool special = false;
 				tags.clear();
-				fill_tagvector(*it, tags, did, special);
+				fill_tagvector(it, tags, did, special);
 				if (!tags.empty()) {
 					ntags[tags] = special;
 				}
@@ -381,7 +379,7 @@ uint32_t Grammar::removeNumericTags(uint32_t s) {
 					u_fflush(ux_stderr);
 				}
 			}
-			Set *ns = allocateSet();
+			Set* ns = allocateSet();
 			ns->type = set->type;
 			ns->line = set->line;
 			ns->name = stringbits[S_GPREFIX].getTerminatedBuffer();
@@ -390,17 +388,17 @@ uint32_t Grammar::removeNumericTags(uint32_t s) {
 			ns->name += 'B';
 			ns->name += '_';
 
-			for (auto it = ntags.begin(); it != ntags.end(); ++it) {
-				if (it->second) {
-					if (it->first.size() == 1 && (it->first[0]->type & T_FAILFAST)) {
-						ns->ff_tags.insert(it->first[0]);
+			for (auto& it : ntags) {
+				if (it.second) {
+					if (it.first.size() == 1 && (it.first[0]->type & T_FAILFAST)) {
+						ns->ff_tags.insert(it.first[0]);
 					}
 					else {
-						trie_insert(ns->trie_special, it->first);
+						trie_insert(ns->trie_special, it.first);
 					}
 				}
 				else {
-					trie_insert(ns->trie, it->first);
+					trie_insert(ns->trie, it.first);
 				}
 			}
 			addSet(ns);
@@ -421,24 +419,24 @@ void Grammar::getTags(const Set& set, std::set<TagVector>& rv) {
 	trie_getTags(set.trie_special, rv, tv);
 }
 
-Rule *Grammar::allocateRule() {
+Rule* Grammar::allocateRule() {
 	return new Rule;
 }
 
-void Grammar::addRule(Rule *rule) {
-	rule->number = rule_by_number.size();
+void Grammar::addRule(Rule* rule) {
+	rule->number = static_cast<uint32_t>(rule_by_number.size());
 	rule_by_number.push_back(rule);
 }
 
-void Grammar::destroyRule(Rule *rule) {
+void Grammar::destroyRule(Rule* rule) {
 	delete rule;
 }
 
-Tag *Grammar::allocateTag() {
+Tag* Grammar::allocateTag() {
 	return new Tag;
 }
 
-Tag *Grammar::allocateTag(const UChar *txt) {
+Tag* Grammar::allocateTag(const UChar* txt) {
 	if (txt[0] == 0) {
 		u_fprintf(ux_stderr, "Error: Empty tag on line %u! Forgot to fill in a ()?\n", lines);
 		CG3Quit(1);
@@ -453,19 +451,19 @@ Tag *Grammar::allocateTag(const UChar *txt) {
 		return it->second;
 	}
 
-	Tag *tag = new Tag();
+	Tag* tag = new Tag();
 	tag->parseTagRaw(txt, this);
 	return addTag(tag);
 }
 
-Tag *Grammar::addTag(Tag *tag) {
+Tag* Grammar::addTag(Tag* tag) {
 	tag->type |= T_GRAMMAR;
 	uint32_t hash = tag->rehash();
 	for (uint32_t seed = 0; seed < 10000; seed++) {
 		uint32_t ih = hash + seed;
 		Taguint32HashMap::iterator it;
 		if ((it = single_tags.find(ih)) != single_tags.end()) {
-			Tag *t = it->second;
+			Tag* t = it->second;
 			if (t == tag) {
 				return tag;
 			}
@@ -483,7 +481,7 @@ Tag *Grammar::addTag(Tag *tag) {
 			tag->seed = seed;
 			hash = tag->rehash();
 			single_tags_list.push_back(tag);
-			tag->number = (uint32_t)single_tags_list.size() - 1;
+			tag->number = static_cast<uint32_t>(single_tags_list.size() - 1);
 			single_tags[hash] = tag;
 			break;
 		}
@@ -491,7 +489,7 @@ Tag *Grammar::addTag(Tag *tag) {
 	return single_tags[hash];
 }
 
-void Grammar::addTagToSet(Tag *rtag, Set *set) {
+void Grammar::addTagToSet(Tag* rtag, Set* set) {
 	if (rtag->type & T_ANY) {
 		set->type |= ST_ANY;
 	}
@@ -507,15 +505,15 @@ void Grammar::addTagToSet(Tag *rtag, Set *set) {
 	}
 }
 
-void Grammar::destroyTag(Tag *tag) {
+void Grammar::destroyTag(Tag* tag) {
 	delete tag;
 }
 
-ContextualTest *Grammar::allocateContextualTest() {
+ContextualTest* Grammar::allocateContextualTest() {
 	return new ContextualTest;
 }
 
-ContextualTest *Grammar::addContextualTest(ContextualTest *t) {
+ContextualTest* Grammar::addContextualTest(ContextualTest* t) {
 	if (t == 0) {
 		return 0;
 	}
@@ -550,7 +548,7 @@ ContextualTest *Grammar::addContextualTest(ContextualTest *t) {
 	return t;
 }
 
-void Grammar::addTemplate(ContextualTest *test, const UChar *name) {
+void Grammar::addTemplate(ContextualTest* test, const UChar* name) {
 	uint32_t cn = hash_value(name);
 	if (templates.find(cn) != templates.end()) {
 		u_fprintf(ux_stderr, "Error: Redefinition attempt for template '%S' on line %u!\n", name, lines);
@@ -559,7 +557,7 @@ void Grammar::addTemplate(ContextualTest *test, const UChar *name) {
 	templates[cn] = test;
 }
 
-void Grammar::addAnchor(const UChar *to, uint32_t at, bool primary) {
+void Grammar::addAnchor(const UChar* to, uint32_t at, bool primary) {
 	uint32_t ah = allocateTag(to)->hash;
 	uint32FlatHashMap::iterator it = anchors.find(ah);
 	if (primary && it != anchors.end()) {
@@ -568,7 +566,7 @@ void Grammar::addAnchor(const UChar *to, uint32_t at, bool primary) {
 	}
 	if (at > rule_by_number.size()) {
 		u_fprintf(ux_stderr, "Warning: No corresponding rule available for anchor '%S' on line %u!\n", to, lines);
-		at = rule_by_number.size();
+		at = static_cast<uint32_t>(rule_by_number.size());
 	}
 	if (it == anchors.end()) {
 		anchors[ah] = at;
@@ -577,8 +575,8 @@ void Grammar::addAnchor(const UChar *to, uint32_t at, bool primary) {
 
 void Grammar::resetStatistics() {
 	total_time = 0;
-	for (uint32_t j = 0; j < rules.size(); j++) {
-		rules[j]->resetStatistics();
+	for (auto r : rules) {
+		r->resetStatistics();
 	}
 }
 
@@ -608,7 +606,7 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 			u_fprintf(ux_stderr, "Error: Static set %S is an alias; only real sets may be made static!\n", sset.c_str());
 			CG3Quit(1);
 		}
-		Set *s = getSet(sh);
+		Set* s = getSet(sh);
 		if (!s) {
 			if (verbosity_level > 0) {
 				u_fprintf(ux_stderr, "Warning: Set %S was not defined, so cannot make it static.\n", sset.c_str());
@@ -661,7 +659,7 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 		}
 		for (auto iter : regex_tags) {
 			UErrorCode status = U_ZERO_ERROR;
-			uregex_setText(iter, titer->tag.c_str(), titer->tag.size(), &status);
+			uregex_setText(iter, titer->tag.c_str(), static_cast<int32_t>(titer->tag.size()), &status);
 			if (status == U_ZERO_ERROR) {
 				if (uregex_find(iter, -1, &status)) {
 					titer->type |= T_TEXTUAL;
@@ -670,7 +668,7 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 		}
 		for (auto iter : icase_tags) {
 			UErrorCode status = U_ZERO_ERROR;
-			if (u_strCaseCompare(titer->tag.c_str(), titer->tag.size(), iter->tag.c_str(), iter->tag.size(), U_FOLD_CASE_DEFAULT, &status) == 0) {
+			if (u_strCaseCompare(titer->tag.c_str(), static_cast<int32_t>(titer->tag.size()), iter->tag.c_str(), static_cast<int32_t>(iter->tag.size()), U_FOLD_CASE_DEFAULT, &status) == 0) {
 				titer->type |= T_TEXTUAL;
 			}
 			if (status != U_ZERO_ERROR) {
@@ -680,23 +678,26 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 		}
 	}
 
-	for (auto it = parentheses.begin(); it != parentheses.end(); ++it) {
-		single_tags[it->first]->markUsed();
-		single_tags[it->second]->markUsed();
+	for (auto& it : parentheses) {
+		single_tags[it.first]->markUsed();
+		single_tags[it.second]->markUsed();
 	}
 
-	for (auto it = preferred_targets.begin(); it != preferred_targets.end(); ++it) {
-		single_tags[*it]->markUsed();
+	for (auto it : preferred_targets) {
+		single_tags[it]->markUsed();
 	}
 
 	for (auto rule : rule_by_number) {
 		if (rule->wordform) {
 			wf_rules.push_back(rule);
 		}
+		if (rule->type == K_PROTECT) {
+			has_protect = true;
+		}
 		if (is_binary) {
 			continue;
 		}
-		Set *s = 0;
+		Set* s = 0;
 		s = getSet(rule->target);
 		s->markUsed(*this);
 		if (rule->childset1) {
@@ -733,12 +734,12 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 		}
 
 		contexts_t tosave;
-		for (auto cntx = contexts.begin(); cntx != contexts.end(); ++cntx) {
-			if (cntx->second->is_used) {
-				tosave[cntx->first] = cntx->second;
+		for (auto& cntx : contexts) {
+			if (cntx.second->is_used) {
+				tosave[cntx.first] = cntx.second;
 				continue;
 			}
-			delete cntx->second;
+			delete cntx.second;
 		}
 		contexts.swap(tosave);
 	}
@@ -765,7 +766,7 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 	}
 
 	for (auto iter_tags : single_tags) {
-		Tag *tag = iter_tags.second;
+		Tag* tag = iter_tags.second;
 		if (tag->tag[0] == mapping_prefix) {
 			tag->type |= T_MAPPING;
 		}
@@ -803,7 +804,7 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 			rules.push_back(rule);
 		}
 		if (rule->target) {
-			Set *set = 0;
+			Set* set = 0;
 			if (is_binary) {
 				set = sets_list[rule->target];
 			}
@@ -825,11 +826,11 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 			continue;
 		}
 		if (rule->childset1) {
-			Set *set = sets_by_contents.find(rule->childset1)->second;
+			Set* set = sets_by_contents.find(rule->childset1)->second;
 			rule->childset1 = set->number;
 		}
 		if (rule->childset2) {
-			Set *set = sets_by_contents.find(rule->childset2)->second;
+			Set* set = sets_by_contents.find(rule->childset2)->second;
 			rule->childset2 = set->number;
 		}
 		if (rule->dep_target) {
@@ -867,7 +868,7 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 				sets_by_name[nhash] = cnum;
 			}
 			else if (cnum != sets_list[sets_by_name.find(nhash)->second]->number) {
-				Set *a = sets_list[sets_by_name.find(nhash)->second];
+				Set* a = sets_list[sets_by_name.find(nhash)->second];
 				if (a->name == to->name) {
 					u_fprintf(ux_stderr, "Error: Static set %S already defined. Redefinition attempted!\n", a->name.c_str());
 					CG3Quit(1);
@@ -918,8 +919,8 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 	while (did) {
 		did = false;
 
-		for (auto cntx : contexts) {
-			ContextualTest *t = cntx.second;
+		for (auto& cntx : contexts) {
+			ContextualTest* t = cntx.second;
 
 			if (nk.count(t)) {
 				continue;
@@ -1004,8 +1005,8 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 	}
 
 	if (used_tags) {
-		for (auto iter_tags = single_tags.begin(); iter_tags != single_tags.end(); ++iter_tags) {
-			Tag *tag = iter_tags->second;
+		for (auto& iter_tags : single_tags) {
+			Tag* tag = iter_tags.second;
 			if (tag->type & T_USED) {
 				UString tmp(tag->toUString(true));
 				u_fprintf(ux_stdout, "%S\n", tmp.c_str());
@@ -1024,7 +1025,7 @@ inline void trie_indexToRule(const trie_t& trie, Grammar& grammar, uint32_t r) {
 	}
 }
 
-void Grammar::indexSetToRule(uint32_t r, Set *s) {
+void Grammar::indexSetToRule(uint32_t r, Set* s) {
 	if (s->type & (ST_SPECIAL | ST_TAG_UNIFY)) {
 		indexTagToRule(tag_any, r);
 		return;
@@ -1033,8 +1034,8 @@ void Grammar::indexSetToRule(uint32_t r, Set *s) {
 	trie_indexToRule(s->trie, *this, r);
 	trie_indexToRule(s->trie_special, *this, r);
 
-	for (uint32_t i = 0; i < s->sets.size(); ++i) {
-		Set *set = sets_list[s->sets[i]];
+	for (auto i : s->sets) {
+		Set* set = sets_list[i];
 		indexSetToRule(r, set);
 	}
 }
@@ -1052,7 +1053,7 @@ inline void trie_indexToSet(const trie_t& trie, Grammar& grammar, uint32_t r) {
 	}
 }
 
-void Grammar::indexSets(uint32_t r, Set *s) {
+void Grammar::indexSets(uint32_t r, Set* s) {
 	if (s->type & (ST_SPECIAL | ST_TAG_UNIFY)) {
 		indexTagToSet(tag_any, r);
 		return;
@@ -1061,8 +1062,8 @@ void Grammar::indexSets(uint32_t r, Set *s) {
 	trie_indexToSet(s->trie, *this, r);
 	trie_indexToSet(s->trie_special, *this, r);
 
-	for (uint32_t i = 0; i < s->sets.size(); ++i) {
-		Set *set = sets_list[s->sets[i]];
+	for (auto i : s->sets) {
+		Set* set = sets_list[i];
 		indexSets(r, set);
 	}
 }
@@ -1074,35 +1075,35 @@ void Grammar::indexTagToSet(uint32_t t, uint32_t r) {
 	sets_by_tag[t].set(r);
 }
 
-void Grammar::setAdjustSets(Set *s) {
+void Grammar::setAdjustSets(Set* s) {
 	if (!(s->type & ST_USED)) {
 		return;
 	}
 	s->type &= ~ST_USED;
 
-	for (uint32_t i = 0; i < s->sets.size(); ++i) {
-		Set *set = sets_by_contents.find(s->sets[i])->second;
-		s->sets[i] = set->number;
+	for (auto& i : s->sets) {
+		Set* set = sets_by_contents.find(i)->second;
+		i = set->number;
 		setAdjustSets(set);
 	}
 }
 
-void Grammar::contextAdjustTarget(ContextualTest *test) {
+void Grammar::contextAdjustTarget(ContextualTest* test) {
 	if (!test->is_used) {
 		return;
 	}
 	test->is_used = false;
 
 	if (test->target) {
-		Set *set = sets_by_contents.find(test->target)->second;
+		Set* set = sets_by_contents.find(test->target)->second;
 		test->target = set->number;
 	}
 	if (test->barrier) {
-		Set *set = sets_by_contents.find(test->barrier)->second;
+		Set* set = sets_by_contents.find(test->barrier)->second;
 		test->barrier = set->number;
 	}
 	if (test->cbarrier) {
-		Set *set = sets_by_contents.find(test->cbarrier)->second;
+		Set* set = sets_by_contents.find(test->cbarrier)->second;
 		test->cbarrier = set->number;
 	}
 
