@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2017, GrammarSoft ApS
+* Copyright (C) 2007-2018, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -36,14 +36,17 @@ class Anchor;
 
 class Grammar {
 public:
-	UFILE *ux_stderr, *ux_stdout;
+	std::ostream* ux_stderr;
+	std::ostream* ux_stdout;
 
 	bool has_dep;
 	bool has_bag_of_tags;
 	bool has_relations;
 	bool has_encl_final;
+	bool has_protect;
 	bool is_binary;
 	bool sub_readings_ltr;
+	bool ordered = false;
 	size_t grammar_size;
 	UChar mapping_prefix;
 	uint32_t lines;
@@ -78,14 +81,14 @@ public:
 	rules_by_set_t rules_by_set;
 	typedef std::unordered_map<uint32_t, uint32IntervalVector> rules_by_tag_t;
 	rules_by_tag_t rules_by_tag;
-	typedef std::unordered_map<uint32_t, boost::dynamic_bitset<> > sets_by_tag_t;
+	typedef std::unordered_map<uint32_t, boost::dynamic_bitset<>> sets_by_tag_t;
 	sets_by_tag_t sets_by_tag;
 
-	uint32IntervalVector *rules_any;
-	boost::dynamic_bitset<> *sets_any;
+	uint32IntervalVector* rules_any;
+	boost::dynamic_bitset<>* sets_any;
 
-	Set *delimiters;
-	Set *soft_delimiters;
+	Set* delimiters;
+	Set* soft_delimiters;
 	uint32_t tag_any;
 	uint32Vector preferred_targets;
 	uint32SortedVector reopen_mappings;
@@ -106,30 +109,30 @@ public:
 	Grammar();
 	~Grammar();
 
-	void addSet(Set *& to);
-	Set *getSet(uint32_t which) const;
-	Set *allocateSet();
-	void destroySet(Set *set);
-	void addSetToList(Set *s);
+	void addSet(Set*& to);
+	Set* getSet(uint32_t which) const;
+	Set* allocateSet();
+	void destroySet(Set* set);
+	void addSetToList(Set* s);
 	void allocateDummySet();
 	uint32_t removeNumericTags(uint32_t s);
 	void getTags(const Set& set, std::set<TagVector>& rv);
 
-	void addAnchor(const UChar *to, uint32_t at, bool primary = false);
+	void addAnchor(const UChar* to, uint32_t at, bool primary = false);
 
-	Tag *allocateTag();
-	Tag *allocateTag(const UChar *tag);
-	Tag *addTag(Tag *tag);
-	void destroyTag(Tag *tag);
-	void addTagToSet(Tag *rtag, Set *set);
+	Tag* allocateTag();
+	Tag* allocateTag(const UChar* tag);
+	Tag* addTag(Tag* tag);
+	void destroyTag(Tag* tag);
+	void addTagToSet(Tag* rtag, Set* set);
 
-	Rule *allocateRule();
-	void addRule(Rule *rule);
-	void destroyRule(Rule *rule);
+	Rule* allocateRule();
+	void addRule(Rule* rule);
+	void destroyRule(Rule* rule);
 
-	ContextualTest *allocateContextualTest();
-	ContextualTest *addContextualTest(ContextualTest *t);
-	void addTemplate(ContextualTest *test, const UChar *name);
+	ContextualTest* allocateContextualTest();
+	ContextualTest* addContextualTest(ContextualTest* t);
+	void addTemplate(ContextualTest* test, const UChar* name);
 
 	void resetStatistics();
 	void reindex(bool unused_sets = false, bool used_tags = false);
@@ -143,7 +146,8 @@ public:
 	void contextAdjustTarget(ContextualTest*);
 };
 
-inline void trie_unserialize(trie_t& trie, FILE *input, Grammar& grammar, uint32_t num_tags) {
+template<typename Stream>
+inline void _trie_unserialize(trie_t& trie, Stream& input, Grammar& grammar, uint32_t num_tags) {
 	for (uint32_t i = 0; i < num_tags; ++i) {
 		uint32_t u32tmp = 0;
 		fread_throw(&u32tmp, sizeof(uint32_t), 1, input);
@@ -164,6 +168,14 @@ inline void trie_unserialize(trie_t& trie, FILE *input, Grammar& grammar, uint32
 		}
 	}
 }
+
+inline void trie_unserialize(trie_t& trie, FILE* input, Grammar& grammar, uint32_t num_tags) {
+	return _trie_unserialize(trie, input, grammar, num_tags);
+}
+
+inline void trie_unserialize(trie_t& trie, std::istream& input, Grammar& grammar, uint32_t num_tags) {
+	return _trie_unserialize(trie, input, grammar, num_tags);
+}
 }
 
 #endif
