@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2017, GrammarSoft ApS
+* Copyright (C) 2007-2018, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -31,7 +31,7 @@
 
 namespace CG3 {
 
-GrammarApplicator::GrammarApplicator(UFILE *ux_err)
+GrammarApplicator::GrammarApplicator(std::ostream& ux_err)
   : always_span(false)
   , apply_mappings(true)
   , apply_corrections(true)
@@ -68,10 +68,9 @@ GrammarApplicator::GrammarApplicator(UFILE *ux_err)
   , section_max_count(0)
   , has_dep(false)
   , dep_highest_seen(0)
-  , gWindow(0)
   , has_relations(false)
   , grammar(0)
-  , ux_stderr(ux_err)
+  , ux_stderr(&ux_err)
   , filebase(0)
   , numLines(0)
   , numWindows(0)
@@ -107,19 +106,16 @@ GrammarApplicator::GrammarApplicator(UFILE *ux_err)
   , unif_sets_firstrun(false)
   , statistics(false)
 {
-	gWindow = new Window(this);
+	gWindow.reset(new Window(this));
 }
 
 GrammarApplicator::~GrammarApplicator() {
-	Taguint32HashMap::iterator iter_stag;
-	for (iter_stag = single_tags.begin(); iter_stag != single_tags.end(); ++iter_stag) {
-		if (iter_stag->second && !(iter_stag->second->type & T_GRAMMAR)) {
-			delete iter_stag->second;
+	for (auto& iter_stag : single_tags) {
+		if (iter_stag.second && !(iter_stag.second->type & T_GRAMMAR)) {
+			delete iter_stag.second;
 		}
 	}
 
-	delete gWindow;
-
 	if (owns_grammar) {
 		delete grammar;
 	}
@@ -140,7 +136,7 @@ void GrammarApplicator::resetIndexes() {
 	index_icase_no.clear();
 }
 
-void GrammarApplicator::setGrammar(Grammar *res) {
+void GrammarApplicator::setGrammar(Grammar* res) {
 	grammar = res;
 	single_tags = grammar->single_tags;
 	tag_begin = addTag(stringbits[S_BEGINTAG].getTerminatedBuffer());
@@ -161,11 +157,8 @@ void GrammarApplicator::index() {
 		return;
 	}
 
-	// ToDo: Remove for real ordered mode
-	for (auto iter : single_tags) {
-		if (iter.second->type & T_REGEXP_LINE) {
-			ordered = true;
-		}
+	if (grammar->ordered) {
+		ordered = true;
 	}
 
 	if (!grammar->before_sections.empty()) {
@@ -190,7 +183,7 @@ void GrammarApplicator::index() {
 	}
 
 	if (sections.empty()) {
-		int32_t smax = (int32_t)grammar->sections.size();
+		int32_t smax = static_cast<int32_t>(grammar->sections.size());
 		for (int32_t i = 0; i < smax; i++) {
 			for (auto r : grammar->rules) {
 				if (r->section < 0 || r->section > i) {
@@ -202,11 +195,11 @@ void GrammarApplicator::index() {
 		}
 	}
 	else {
-		numsections = sections.size();
+		numsections = static_cast<uint32_t>(sections.size());
 		for (uint32_t n = 0; n < numsections; n++) {
 			for (uint32_t e = 0; e <= n; e++) {
 				for (auto r : grammar->rules) {
-					if (r->section != (int32_t)sections[e] - 1) {
+					if (r->section != static_cast<int32_t>(sections[e]) - 1) {
 						continue;
 					}
 					uint32IntervalVector& m = runsections[n];
@@ -247,14 +240,14 @@ void GrammarApplicator::disableStatistics() {
 	statistics = false;
 }
 
-Tag *GrammarApplicator::addTag(Tag *tag) {
+Tag* GrammarApplicator::addTag(Tag* tag) {
 	uint32_t hash = tag->rehash();
 	uint32_t seed = 0;
 	for (; seed < 10000; seed++) {
 		uint32_t ih = hash + seed;
 		Taguint32HashMap::iterator it;
 		if ((it = single_tags.find(ih)) != single_tags.end()) {
-			Tag *t = it->second;
+			Tag* t = it->second;
 			if (t == tag) {
 				return tag;
 			}
@@ -278,14 +271,14 @@ Tag *GrammarApplicator::addTag(Tag *tag) {
 	return single_tags[hash];
 }
 
-Tag *GrammarApplicator::addTag(const UChar *txt, bool vstr) {
+Tag* GrammarApplicator::addTag(const UChar* txt, bool vstr) {
 	Taguint32HashMap::iterator it;
 	uint32_t thash = hash_value(txt);
 	if ((it = single_tags.find(thash)) != single_tags.end() && !it->second->tag.empty() && u_strcmp(it->second->tag.c_str(), txt) == 0) {
 		return it->second;
 	}
 
-	Tag *tag = 0;
+	Tag* tag = 0;
 	if (vstr) {
 		tag = ::CG3::parseTag(txt, 0, *this);
 	}
@@ -304,7 +297,7 @@ Tag *GrammarApplicator::addTag(const UChar *txt, bool vstr) {
 				}
 				for (auto iter : grammar->regex_tags) {
 					UErrorCode status = U_ZERO_ERROR;
-					uregex_setText(iter, titer.second->tag.c_str(), titer.second->tag.size(), &status);
+					uregex_setText(iter, titer.second->tag.c_str(), static_cast<int32_t>(titer.second->tag.size()), &status);
 					if (status == U_ZERO_ERROR) {
 						if (uregex_find(iter, -1, &status)) {
 							titer.second->type |= T_TEXTUAL;
@@ -323,7 +316,7 @@ Tag *GrammarApplicator::addTag(const UChar *txt, bool vstr) {
 				}
 				for (auto iter : grammar->icase_tags) {
 					UErrorCode status = U_ZERO_ERROR;
-					if (u_strCaseCompare(titer.second->tag.c_str(), titer.second->tag.size(), iter->tag.c_str(), iter->tag.size(), U_FOLD_CASE_DEFAULT, &status) == 0) {
+					if (u_strCaseCompare(titer.second->tag.c_str(), static_cast<int32_t>(titer.second->tag.size()), iter->tag.c_str(), static_cast<int32_t>(iter->tag.size()), U_FOLD_CASE_DEFAULT, &status) == 0) {
 						titer.second->type |= T_TEXTUAL;
 						reflow = true;
 					}
@@ -341,15 +334,14 @@ Tag *GrammarApplicator::addTag(const UChar *txt, bool vstr) {
 	return tag;
 }
 
-
-Tag *GrammarApplicator::addTag(const UString& txt, bool vstr) {
-	assert(txt.length() && "addTag() will not work with empty strings.");
+Tag* GrammarApplicator::addTag(const UString& txt, bool vstr) {
+	assert(txt.size() && "addTag() will not work with empty strings.");
 	return addTag(txt.c_str(), vstr);
 }
 
-void GrammarApplicator::printTrace(UFILE *output, uint32_t hit_by) {
+void GrammarApplicator::printTrace(std::ostream& output, uint32_t hit_by) {
 	if (hit_by < grammar->rule_by_number.size()) {
-		const Rule *r = grammar->rule_by_number[hit_by];
+		const Rule* r = grammar->rule_by_number[hit_by];
 		u_fprintf(output, "%S", keywords[r->type].getTerminatedBuffer());
 		if (r->type == K_ADDRELATION || r->type == K_SETRELATION || r->type == K_REMRELATION || r->type == K_ADDRELATIONS || r->type == K_SETRELATIONS || r->type == K_REMRELATIONS) {
 			u_fprintf(output, "(%S", r->maplist->getNonEmpty().begin()->first->tag.c_str());
@@ -372,7 +364,7 @@ void GrammarApplicator::printTrace(UFILE *output, uint32_t hit_by) {
 	}
 }
 
-void GrammarApplicator::printReading(const Reading *reading, UFILE *output, size_t sub) {
+void GrammarApplicator::printReading(const Reading* reading, std::ostream& output, size_t sub) {
 	if (reading->noprint) {
 		return;
 	}
@@ -406,7 +398,7 @@ void GrammarApplicator::printReading(const Reading *reading, UFILE *output, size
 			}
 			unique.insert(tter);
 		}
-		const Tag *tag = single_tags[tter];
+		const Tag* tag = single_tags[tter];
 		if (tag->type & T_DEPENDENCY && has_dep && !dep_original) {
 			continue;
 		}
@@ -420,7 +412,7 @@ void GrammarApplicator::printReading(const Reading *reading, UFILE *output, size
 		if (!reading->parent->dep_self) {
 			reading->parent->dep_self = reading->parent->global_number;
 		}
-		const Cohort *pr = 0;
+		const Cohort* pr = 0;
 		pr = reading->parent;
 		if (reading->parent->dep_parent != DEP_NO_PARENT) {
 			if (reading->parent->dep_parent == 0) {
@@ -433,7 +425,7 @@ void GrammarApplicator::printReading(const Reading *reading, UFILE *output, size
 
 		constexpr UChar local_utf_pattern[] = { ' ', '#', '%', 'u', L'\u2192', '%', 'u', 0 };
 		constexpr UChar local_latin_pattern[] = { ' ', '#', '%', 'u', '-', '>', '%', 'u', 0 };
-		const UChar *pattern = local_latin_pattern;
+		const UChar* pattern = local_latin_pattern;
 		if (unicode_tags) {
 			pattern = local_utf_pattern;
 		}
@@ -490,7 +482,7 @@ void GrammarApplicator::printReading(const Reading *reading, UFILE *output, size
 	}
 }
 
-void GrammarApplicator::printCohort(Cohort *cohort, UFILE *output) {
+void GrammarApplicator::printCohort(Cohort* cohort, std::ostream& output) {
 	constexpr UChar ws[] = { ' ', '\t', 0 };
 
 	if (cohort->local_number == 0) {
@@ -510,7 +502,7 @@ void GrammarApplicator::printCohort(Cohort *cohort, UFILE *output) {
 			if (tter == cohort->wordform->hash) {
 				continue;
 			}
-			const Tag *tag = single_tags[tter];
+			const Tag* tag = single_tags[tter];
 			u_fprintf(output, " %S", tag->tag.c_str());
 		}
 	}
@@ -535,7 +527,7 @@ void GrammarApplicator::printCohort(Cohort *cohort, UFILE *output) {
 removed:
 	if (!cohort->text.empty() && cohort->text.find_first_not_of(ws) != UString::npos) {
 		u_fprintf(output, "%S", cohort->text.c_str());
-		if (!ISNL(cohort->text[cohort->text.length() - 1])) {
+		if (!ISNL(cohort->text[cohort->text.size() - 1])) {
 			u_fputc('\n', output);
 		}
 	}
@@ -545,13 +537,13 @@ removed:
 	}
 }
 
-void GrammarApplicator::printSingleWindow(SingleWindow *window, UFILE *output) {
+void GrammarApplicator::printSingleWindow(SingleWindow* window, std::ostream& output) {
 	for (auto var : window->variables_output) {
-		Tag *key = single_tags[var];
+		Tag* key = single_tags[var];
 		auto iter = window->variables_set.find(var);
 		if (iter != window->variables_set.end()) {
 			if (iter->second != grammar->tag_any) {
-				Tag *value = single_tags[iter->second];
+				Tag* value = single_tags[iter->second];
 				u_fprintf(output, "%S%S=%S>\n", stringbits[S_CMD_SETVAR].getTerminatedBuffer(), key->tag.c_str(), value->tag.c_str());
 			}
 			else {
@@ -565,21 +557,21 @@ void GrammarApplicator::printSingleWindow(SingleWindow *window, UFILE *output) {
 
 	if (!window->text.empty()) {
 		u_fprintf(output, "%S", window->text.c_str());
-		if (!ISNL(window->text[window->text.length() - 1])) {
+		if (!ISNL(window->text[window->text.size() - 1])) {
 			u_fputc('\n', output);
 		}
 	}
 
-	uint32_t cs = (uint32_t)window->cohorts.size();
+	uint32_t cs = static_cast<uint32_t>(window->cohorts.size());
 	for (uint32_t c = 0; c < cs; c++) {
-		Cohort *cohort = window->cohorts[c];
+		Cohort* cohort = window->cohorts[c];
 		printCohort(cohort, output);
 	}
 	u_fputc('\n', output);
 	u_fflush(output);
 }
 
-void GrammarApplicator::pipeOutReading(const Reading *reading, std::ostream& output) {
+void GrammarApplicator::pipeOutReading(const Reading* reading, std::ostream& output) {
 	std::ostringstream ss;
 
 	uint32_t flags = 0;
@@ -605,7 +597,7 @@ void GrammarApplicator::pipeOutReading(const Reading *reading, std::ostream& out
 		if (tter == reading->baseform || tter == reading->parent->wordform->hash) {
 			continue;
 		}
-		const Tag *tag = single_tags.find(tter)->second;
+		const Tag* tag = single_tags.find(tter)->second;
 		if (tag->type & T_DEPENDENCY && has_dep) {
 			continue;
 		}
@@ -617,20 +609,20 @@ void GrammarApplicator::pipeOutReading(const Reading *reading, std::ostream& out
 		if (tter == reading->baseform || tter == reading->parent->wordform->hash) {
 			continue;
 		}
-		const Tag *tag = single_tags.find(tter)->second;
+		const Tag* tag = single_tags.find(tter)->second;
 		if (tag->type & T_DEPENDENCY && has_dep) {
 			continue;
 		}
 		writeUTF8String(ss, tag->tag);
 	}
 
-	std::string str = ss.str();
-	cs = str.length();
+	const auto& str = ss.str();
+	cs = static_cast<uint32_t>(str.size());
 	writeRaw(output, cs);
-	output.write(str.c_str(), str.length());
+	output.write(str.c_str(), str.size());
 }
 
-void GrammarApplicator::pipeOutCohort(const Cohort *cohort, std::ostream& output) {
+void GrammarApplicator::pipeOutCohort(const Cohort* cohort, std::ostream& output) {
 	std::ostringstream ss;
 
 	writeRaw(ss, cohort->global_number);
@@ -650,7 +642,7 @@ void GrammarApplicator::pipeOutCohort(const Cohort *cohort, std::ostream& output
 
 	writeUTF8String(ss, cohort->wordform->tag);
 
-	uint32_t cs = cohort->readings.size();
+	uint32_t cs = static_cast<uint32_t>(cohort->readings.size());
 	writeRaw(ss, cs);
 	for (auto rter1 : cohort->readings) {
 		pipeOutReading(rter1, ss);
@@ -659,10 +651,10 @@ void GrammarApplicator::pipeOutCohort(const Cohort *cohort, std::ostream& output
 		writeUTF8String(ss, cohort->text);
 	}
 
-	std::string str = ss.str();
-	cs = str.length();
+	const auto& str = ss.str();
+	cs = static_cast<uint32_t>(str.size());
 	writeRaw(output, cs);
-	output.write(str.c_str(), str.length());
+	output.write(str.c_str(), str.size());
 }
 
 void GrammarApplicator::pipeOutSingleWindow(const SingleWindow& window, Process& output) {
@@ -677,15 +669,15 @@ void GrammarApplicator::pipeOutSingleWindow(const SingleWindow& window, Process&
 		pipeOutCohort(window.cohorts[c], ss);
 	}
 
-	std::string str = ss.str();
-	cs = str.length();
+	const auto& str = ss.str();
+	cs = static_cast<uint32_t>(str.size());
 	writeRaw(output, cs);
-	output.write(str.c_str(), str.length());
+	output.write(str.c_str(), str.size());
 
 	output.flush();
 }
 
-void GrammarApplicator::pipeInReading(Reading *reading, Process& input, bool force) {
+void GrammarApplicator::pipeInReading(Reading* reading, Process& input, bool force) {
 	uint32_t cs = 0;
 	readRaw(input, cs);
 	if (debug_level > 1) {
@@ -713,7 +705,7 @@ void GrammarApplicator::pipeInReading(Reading *reading, Process& input, bool for
 	if (flags & (1 << 3)) {
 		UString str = readUTF8String(ss);
 		if (str != single_tags.find(reading->baseform)->second->tag) {
-			Tag *tag = addTag(str);
+			Tag* tag = addTag(str);
 			reading->baseform = tag->hash;
 		}
 		if (debug_level > 1) {
@@ -737,7 +729,7 @@ void GrammarApplicator::pipeInReading(Reading *reading, Process& input, bool for
 
 	for (size_t i = 0; i < cs; ++i) {
 		UString str = readUTF8String(ss);
-		Tag *tag = addTag(str);
+		Tag* tag = addTag(str);
 		reading->tags_list.push_back(tag->hash);
 		if (debug_level > 1) {
 			u_fprintf(ux_stderr, "DEBUG: tag %S\n", tag->tag.c_str());
@@ -747,7 +739,7 @@ void GrammarApplicator::pipeInReading(Reading *reading, Process& input, bool for
 	reflowReading(*reading);
 }
 
-void GrammarApplicator::pipeInCohort(Cohort *cohort, Process& input) {
+void GrammarApplicator::pipeInCohort(Cohort* cohort, Process& input) {
 	uint32_t cs = 0;
 	readRaw(input, cs);
 	if (debug_level > 1) {
@@ -779,7 +771,7 @@ void GrammarApplicator::pipeInCohort(Cohort *cohort, Process& input) {
 	bool force_readings = false;
 	UString str = readUTF8String(input);
 	if (str != cohort->wordform->tag) {
-		Tag *tag = addTag(str);
+		Tag* tag = addTag(str);
 		cohort->wordform = tag;
 		force_readings = true;
 		if (debug_level > 1) {
@@ -829,7 +821,7 @@ void GrammarApplicator::pipeInSingleWindow(SingleWindow& window, Process& input)
 	}
 }
 
-void GrammarApplicator::error(const char *str, const UChar *p) {
+void GrammarApplicator::error(const char* str, const UChar* p) {
 	(void)p;
 	if (current_rule && current_rule->line) {
 		constexpr UChar buf[] = { 'R', 'T', ' ', 'R', 'U', 'L', 'E', 0 };
@@ -841,7 +833,7 @@ void GrammarApplicator::error(const char *str, const UChar *p) {
 	}
 }
 
-void GrammarApplicator::error(const char *str, const char *s, const UChar *p) {
+void GrammarApplicator::error(const char* str, const char* s, const UChar* p) {
 	(void)p;
 	if (current_rule && current_rule->line) {
 		constexpr UChar buf[] = { 'R', 'T', ' ', 'R', 'U', 'L', 'E', 0 };
@@ -853,7 +845,7 @@ void GrammarApplicator::error(const char *str, const char *s, const UChar *p) {
 	}
 }
 
-void GrammarApplicator::error(const char *str, const UChar *s, const UChar *p) {
+void GrammarApplicator::error(const char* str, const UChar* s, const UChar* p) {
 	(void)p;
 	if (current_rule && current_rule->line) {
 		constexpr UChar buf[] = { 'R', 'T', ' ', 'R', 'U', 'L', 'E', 0 };
@@ -865,7 +857,7 @@ void GrammarApplicator::error(const char *str, const UChar *s, const UChar *p) {
 	}
 }
 
-void GrammarApplicator::error(const char *str, const char *s, const UChar *S, const UChar *p) {
+void GrammarApplicator::error(const char* str, const char* s, const UChar* S, const UChar* p) {
 	(void)p;
 	if (current_rule && current_rule->line) {
 		constexpr UChar buf[] = { 'R', 'T', ' ', 'R', 'U', 'L', 'E', 0 };
