@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2017, GrammarSoft ApS
+* Copyright (C) 2007-2018, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -29,12 +29,12 @@
 
 namespace CG3 {
 
-void GrammarApplicator::initEmptySingleWindow(SingleWindow *cSWindow) {
-	Cohort *cCohort = alloc_cohort(cSWindow);
+void GrammarApplicator::initEmptySingleWindow(SingleWindow* cSWindow) {
+	Cohort* cCohort = alloc_cohort(cSWindow);
 	cCohort->global_number = 0;
 	cCohort->wordform = tag_begin;
 
-	Reading *cReading = alloc_reading(cCohort);
+	Reading* cReading = alloc_reading(cCohort);
 	cReading->baseform = begintag;
 	insert_if_exists(cReading->parent->possible_sets, grammar->sets_any);
 	addTagToReading(*cReading, begintag);
@@ -44,8 +44,8 @@ void GrammarApplicator::initEmptySingleWindow(SingleWindow *cSWindow) {
 	cSWindow->appendCohort(cCohort);
 }
 
-Reading *GrammarApplicator::initEmptyCohort(Cohort& cCohort) {
-	Reading *cReading = alloc_reading(&cCohort);
+Reading* GrammarApplicator::initEmptyCohort(Cohort& cCohort) {
+	Reading* cReading = alloc_reading(&cCohort);
 	if (allow_magic_readings) {
 		cReading->baseform = makeBaseFromWord(cCohort.wordform)->hash;
 	}
@@ -60,7 +60,10 @@ Reading *GrammarApplicator::initEmptyCohort(Cohort& cCohort) {
 	return cReading;
 }
 
-void GrammarApplicator::runGrammarOnText(istream& input, UFILE *output) {
+void GrammarApplicator::runGrammarOnText(std::istream& input, std::ostream& output) {
+	ux_stdin = &input;
+	ux_stdout = &output;
+
 	if (!input.good()) {
 		u_fprintf(ux_stderr, "Error: Input is null - nothing to parse!\n");
 		CG3Quit(1);
@@ -88,7 +91,7 @@ void GrammarApplicator::runGrammarOnText(istream& input, UFILE *output) {
 	}
 
 	std::vector<UChar> line(1024, 0);
-	std::vector<UChar> cleaned(line.size(), 0);
+	std::vector<UChar> cleaned(line.size() + 1, 0);
 	bool ignoreinput = false;
 	bool did_soft_lookback = false;
 
@@ -97,13 +100,13 @@ void GrammarApplicator::runGrammarOnText(istream& input, UFILE *output) {
 	uint32_t resetAfter = ((num_windows + 4) * 2 + 1);
 	uint32_t lines = 0;
 
-	SingleWindow *cSWindow = 0;
-	Cohort *cCohort = 0;
-	Reading *cReading = 0;
+	SingleWindow* cSWindow = 0;
+	Cohort* cCohort = 0;
+	Reading* cReading = 0;
 
-	SingleWindow *lSWindow = 0;
-	Cohort *lCohort = 0;
-	Reading *lReading = 0;
+	SingleWindow* lSWindow = 0;
+	Cohort* lCohort = 0;
+	Reading* lReading = 0;
 
 	gWindow->window_span = num_windows;
 	gtimer = getticks();
@@ -113,14 +116,16 @@ void GrammarApplicator::runGrammarOnText(istream& input, UFILE *output) {
 	uint32FlatHashSet variables_rem;
 	uint32SortedVector variables_output;
 
-	std::vector<std::pair<size_t, Reading*> > indents;
+	std::vector<std::pair<size_t, Reading*>> indents;
 	all_mappings_t all_mappings;
 
+	ux_stripBOM(input);
+
 	while (!input.eof()) {
 		++lines;
 		size_t offset = 0, packoff = 0;
 		// Read as much of the next line as will fit in the current buffer
-		while (input.gets(&line[offset], line.size() - offset - 1)) {
+		while (u_fgets(&line[offset], static_cast<int32_t>(line.size() - offset - 1), input)) {
 			// Copy the segment just read to cleaned
 			for (size_t i = offset; i < line.size(); ++i) {
 				// Only copy one space character, regardless of how many are in input
@@ -157,8 +162,11 @@ void GrammarApplicator::runGrammarOnText(istream& input, UFILE *output) {
 			cleaned[packoff - 1] = 0;
 			--packoff;
 		}
-		if (!ignoreinput && cleaned[0] == '"' && cleaned[1] == '<') {
-			UChar *space = &cleaned[0];
+		if (ignoreinput) {
+			goto istext;
+		}
+		if (cleaned[0] == '"' && cleaned[1] == '<') {
+			UChar* space = &cleaned[0];
 			if (space[0] == '"' && space[1] == '<') {
 				++space;
 				SKIPTO_NOSPAN(space, '"');
@@ -184,7 +192,7 @@ void GrammarApplicator::runGrammarOnText(istream& input, UFILE *output) {
 				reverse_foreach (iter, cSWindow->cohorts) {
 					if (doesSetMatchCohortNormal(**iter, grammar->soft_delimiters->number)) {
 						did_soft_lookback = false;
-						Cohort *cohort = delimitAt(*cSWindow, *iter);
+						Cohort* cohort = delimitAt(*cSWindow, *iter);
 						cSWindow = cohort->parent->next;
 						if (cCohort) {
 							cCohort->parent = cSWindow;
@@ -252,13 +260,7 @@ void GrammarApplicator::runGrammarOnText(istream& input, UFILE *output) {
 				splitAllMappings(all_mappings, *cCohort, true);
 				cSWindow->appendCohort(cCohort);
 			}
-			if (gWindow->next.size() > num_windows) {
-				while (!gWindow->previous.empty() && gWindow->previous.size() > num_windows) {
-					SingleWindow *tmp = gWindow->previous.front();
-					printSingleWindow(tmp, output);
-					free_swindow(tmp);
-					gWindow->previous.erase(gWindow->previous.begin());
-				}
+			if (gWindow->next.size() > num_windows + 1) {
 				gWindow->shuffleWindowsDown();
 				runGrammarOnWindow();
 				if (numWindows % resetAfter == 0) {
@@ -283,14 +285,14 @@ void GrammarApplicator::runGrammarOnText(istream& input, UFILE *output) {
 				addTagToReading(*cCohort->wread, cCohort->wordform);
 				while (space[0]) {
 					SKIPWS(space, 0, 0, true);
-					UChar *n = space;
+					UChar* n = space;
 					if (*n == '"') {
 						++n;
 						SKIPTO_NOSPAN(n, '"');
 					}
 					SKIPTOWS(n, 0, true, true);
 					n[0] = 0;
-					Tag *tag = addTag(space);
+					Tag* tag = addTag(space);
 					addTagToReading(*cCohort->wread, tag);
 					space = ++n;
 				}
@@ -321,8 +323,8 @@ void GrammarApplicator::runGrammarOnText(istream& input, UFILE *output) {
 			insert_if_exists(cReading->parent->possible_sets, grammar->sets_any);
 			addTagToReading(*cReading, cCohort->wordform);
 
-			UChar *space = &cleaned[1];
-			UChar *base = space;
+			UChar* space = &cleaned[1];
+			UChar* base = space;
 			if (*space == '"') {
 				++space;
 				SKIPTO_NOSPAN(space, '"');
@@ -346,7 +348,7 @@ void GrammarApplicator::runGrammarOnText(istream& input, UFILE *output) {
 				space[0] = 0;
 				++space;
 				if (base && base[0]) {
-					Tag *tag = addTag(base);
+					Tag* tag = addTag(base);
 					if (tag->type & T_MAPPING || tag->tag[0] == grammar->mapping_prefix) {
 						all_mappings[cReading].push_back(tag);
 					}
@@ -361,7 +363,7 @@ void GrammarApplicator::runGrammarOnText(istream& input, UFILE *output) {
 				}
 			}
 			if (base && base[0]) {
-				Tag *tag = addTag(base);
+				Tag* tag = addTag(base);
 				if (tag->type & T_MAPPING || tag->tag[0] == grammar->mapping_prefix) {
 					all_mappings[cReading].push_back(tag);
 				}
@@ -426,7 +428,7 @@ void GrammarApplicator::runGrammarOnText(istream& input, UFILE *output) {
 			}
 		}
 		else {
-			if (!ignoreinput && cleaned[0] == ' ' && cleaned[1] == '"') {
+			if (cleaned[0] == ' ' && cleaned[1] == '"') {
 				if (verbosity_level > 0) {
 					u_fprintf(ux_stderr, "Warning: %S on line %u looked like a reading but there was no containing cohort - treated as plain text.\n", &cleaned[0], numLines);
 					u_fflush(ux_stderr);
@@ -452,12 +454,6 @@ void GrammarApplicator::runGrammarOnText(istream& input, UFILE *output) {
 						cSWindow = lSWindow = 0;
 					}
 					while (!gWindow->next.empty()) {
-						while (!gWindow->previous.empty() && gWindow->previous.size() > num_windows) {
-							SingleWindow *tmp = gWindow->previous.front();
-							printSingleWindow(tmp, output);
-							free_swindow(tmp);
-							gWindow->previous.erase(gWindow->previous.begin());
-						}
 						gWindow->shuffleWindowsDown();
 						runGrammarOnWindow();
 						if (numWindows % resetAfter == 0) {
@@ -470,7 +466,7 @@ void GrammarApplicator::runGrammarOnText(istream& input, UFILE *output) {
 					}
 					gWindow->shuffleWindowsDown();
 					while (!gWindow->previous.empty()) {
-						SingleWindow *tmp = gWindow->previous.front();
+						SingleWindow* tmp = gWindow->previous.front();
 						printSingleWindow(tmp, output);
 						free_swindow(tmp);
 						gWindow->previous.erase(gWindow->previous.begin());
@@ -507,11 +503,11 @@ void GrammarApplicator::runGrammarOnText(istream& input, UFILE *output) {
 					cleaned[packoff - 1] = 0;
 					line[0] = 0;
 
-					UChar *s = &cleaned[stringbits[S_CMD_SETVAR].length()];
-					UChar *c = u_strchr(s, ',');
-					UChar *d = u_strchr(s, '=');
+					UChar* s = &cleaned[stringbits[S_CMD_SETVAR].length()];
+					UChar* c = u_strchr(s, ',');
+					UChar* d = u_strchr(s, '=');
 					if (c == 0 && d == 0) {
-						Tag *tag = addTag(s);
+						Tag* tag = addTag(s);
 						variables_set[tag->hash] = grammar->tag_any;
 						variables_rem.erase(tag->hash);
 						variables_output.insert(tag->hash);
@@ -583,8 +579,8 @@ void GrammarApplicator::runGrammarOnText(istream& input, UFILE *output) {
 					cleaned[packoff - 1] = 0;
 					line[0] = 0;
 
-					UChar *s = &cleaned[stringbits[S_CMD_REMVAR].length()];
-					UChar *c = u_strchr(s, ',');
+					UChar* s = &cleaned[stringbits[S_CMD_REMVAR].length()];
+					UChar* c = u_strchr(s, ',');
 					uint32_t a = 0;
 					while (c && *c) {
 						c[0] = 0;
@@ -638,12 +634,6 @@ void GrammarApplicator::runGrammarOnText(istream& input, UFILE *output) {
 		cSWindow = 0;
 	}
 	while (!gWindow->next.empty()) {
-		while (!gWindow->previous.empty() && gWindow->previous.size() > num_windows) {
-			SingleWindow *tmp = gWindow->previous.front();
-			printSingleWindow(tmp, output);
-			free_swindow(tmp);
-			gWindow->previous.erase(gWindow->previous.begin());
-		}
 		gWindow->shuffleWindowsDown();
 		runGrammarOnWindow();
 		if (verbosity_level > 0) {
@@ -654,7 +644,7 @@ void GrammarApplicator::runGrammarOnText(istream& input, UFILE *output) {
 
 	gWindow->shuffleWindowsDown();
 	while (!gWindow->previous.empty()) {
-		SingleWindow *tmp = gWindow->previous.front();
+		SingleWindow* tmp = gWindow->previous.front();
 		printSingleWindow(tmp, output);
 		free_swindow(tmp);
 		gWindow->previous.erase(gWindow->previous.begin());
