@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2017, GrammarSoft ApS
+* Copyright (C) 2007-2018, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -36,11 +36,13 @@ typedef std::vector<Reading*> ReadingList;
 
 class Reading {
 public:
-	bool mapped;
-	bool deleted;
-	bool noprint;
-	bool matched_target;
-	bool matched_tests;
+	uint8_t mapped : 1;
+	uint8_t deleted : 1;
+	uint8_t noprint : 1;
+	uint8_t matched_target : 1;
+	uint8_t matched_tests : 1;
+	uint8_t immutable : 1;
+
 	uint32_t baseform;
 	uint32_t hash;
 	uint32_t hash_plain;
@@ -48,9 +50,9 @@ public:
 	uint32Bloomish tags_bloom;
 	uint32Bloomish tags_plain_bloom;
 	uint32Bloomish tags_textual_bloom;
-	Tag *mapping;
-	Cohort *parent;
-	Reading *next;
+	Tag* mapping;
+	Cohort* parent;
+	Reading* next;
 	uint32Vector hit_by;
 	typedef uint32Vector tags_list_t;
 	tags_list_t tags_list;
@@ -59,26 +61,26 @@ public:
 	uint32SortedVector tags_textual;
 	typedef bc::flat_map<uint32_t, Tag*> tags_numerical_t;
 	tags_numerical_t tags_numerical;
-	
+
 	// ToDo: Remove for real ordered mode
 	UString tags_string;
 	uint32_t tags_string_hash = 0;
 
-	Reading(Cohort *p = 0);
+	Reading(Cohort* p = 0);
 	Reading(const Reading& r);
 	~Reading();
 	void clear();
 
-	Reading *allocateReading(Cohort *p);
-	Reading *allocateReading(const Reading& r);
+	Reading* allocateReading(Cohort* p);
+	Reading* allocateReading(const Reading& r);
 
 	uint32_t rehash();
-	static bool cmp_number(Reading *a, Reading *b);
+	static bool cmp_number(Reading* a, Reading* b);
 };
 
-Reading *alloc_reading(Cohort *p = 0);
-Reading *alloc_reading(const Reading& r);
-void free_reading(Reading *r);
+Reading* alloc_reading(Cohort* p = 0);
+Reading* alloc_reading(const Reading& r);
+void free_reading(Reading* r);
 }
 
 #endif
