@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2017, GrammarSoft ApS
+* Copyright (C) 2007-2018, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -26,7 +26,7 @@
 
 namespace CG3 {
 
-Window::Window(GrammarApplicator *p)
+Window::Window(GrammarApplicator* p)
   : parent(p)
   , cohort_counter(1)
   , window_counter(0)
@@ -36,28 +36,27 @@ Window::Window(GrammarApplicator *p)
 }
 
 Window::~Window() {
-	SingleWindowCont::iterator iter;
-	for (iter = previous.begin(); iter != previous.end(); iter++) {
-		delete *iter;
+	for (auto iter : previous) {
+		delete iter;
 	}
 
 	delete current;
 	current = 0;
 
-	for (iter = next.begin(); iter != next.end(); iter++) {
-		delete *iter;
+	for (auto iter : next) {
+		delete iter;
 	}
 }
 
-SingleWindow *Window::allocSingleWindow() {
-	SingleWindow *swindow = alloc_swindow(this);
+SingleWindow* Window::allocSingleWindow() {
+	SingleWindow* swindow = alloc_swindow(this);
 	window_counter++;
 	swindow->number = window_counter;
 	return swindow;
 }
 
-SingleWindow *Window::allocPushSingleWindow() {
-	SingleWindow *swindow = alloc_swindow(this);
+SingleWindow* Window::allocPushSingleWindow() {
+	SingleWindow* swindow = alloc_swindow(this);
 	window_counter++;
 	swindow->number = window_counter;
 	if (!next.empty()) {
@@ -72,8 +71,8 @@ SingleWindow *Window::allocPushSingleWindow() {
 	return swindow;
 }
 
-SingleWindow *Window::allocAppendSingleWindow() {
-	SingleWindow *swindow = alloc_swindow(this);
+SingleWindow* Window::allocAppendSingleWindow() {
+	SingleWindow* swindow = alloc_swindow(this);
 	window_counter++;
 	swindow->number = window_counter;
 	if (!next.empty()) {
@@ -99,7 +98,7 @@ void Window::shuffleWindowsDown() {
 }
 
 void Window::rebuildSingleWindowLinks() {
-	SingleWindow *sWindow = 0;
+	SingleWindow* sWindow = 0;
 
 	for (auto iter : previous) {
 		iter->previous = sWindow;
@@ -131,7 +130,7 @@ void Window::rebuildSingleWindowLinks() {
 }
 
 void Window::rebuildCohortLinks() {
-	SingleWindow *sWindow = 0;
+	SingleWindow* sWindow = 0;
 	if (!previous.empty()) {
 		sWindow = previous.front();
 	}
@@ -142,7 +141,7 @@ void Window::rebuildCohortLinks() {
 		sWindow = next.front();
 	}
 
-	Cohort *prev = 0;
+	Cohort* prev = 0;
 	while (sWindow) {
 		for (auto citer : sWindow->cohorts) {
 			citer->prev = prev;
