@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2017, GrammarSoft ApS
+* Copyright (C) 2007-2018, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -42,10 +42,10 @@ public:
 	typedef typename std::vector<value_type_real> container;
 	typedef typename container::size_type size_type;
 
-	class const_iterator : public std::iterator<std::bidirectional_iterator_tag, value_type> {
+	class const_iterator {
 	private:
 		friend class flat_unordered_map;
-		const flat_unordered_map *fus;
+		const flat_unordered_map* fus;
 		size_t i;
 
 	public:
@@ -109,7 +109,7 @@ public:
 			return fus->elements[i];
 		}
 
-		const value_type_real *operator->() const {
+		const value_type_real* operator->() const {
 			return &fus->elements[i];
 		}
 	};
