@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2017, GrammarSoft ApS
+* Copyright (C) 2007-2018, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -112,10 +112,10 @@ UOption options[] = {
 	UOPTION_DEF_D("stdout",              'O', UOPT_REQUIRES_ARG, "file to print output to instead of stdout"),
 	UOPTION_DEF_D("stderr",              'E', UOPT_REQUIRES_ARG, "file to print errors to instead of stderr"),
 
-	UOPTION_DEF_D("codepage-all",        'C', UOPT_REQUIRES_ARG, "codepage to use for grammar, input, and output streams; defaults to UTF-8"),
-	UOPTION_DEF_D("codepage-grammar",      0, UOPT_REQUIRES_ARG, "codepage to use for grammar; overrides --codepage-all"),
-	UOPTION_DEF_D("codepage-input",        0, UOPT_REQUIRES_ARG, "codepage to use for input; overrides --codepage-all"),
-	UOPTION_DEF_D("codepage-output",       0, UOPT_REQUIRES_ARG, "codepage to use for output and errors; overrides --codepage-all"),
+	UOPTION_DEF_D("codepage-all",        'C', UOPT_REQUIRES_ARG, nullptr),
+	UOPTION_DEF_D("codepage-grammar",      0, UOPT_REQUIRES_ARG, nullptr),
+	UOPTION_DEF_D("codepage-input",        0, UOPT_REQUIRES_ARG, nullptr),
+	UOPTION_DEF_D("codepage-output",       0, UOPT_REQUIRES_ARG, nullptr),
 
 	UOPTION_DEF_D("no-mappings",           0, UOPT_NO_ARG,       "disables all MAP, ADD, and REPLACE rules"),
 	UOPTION_DEF_D("no-corrections",        0, UOPT_NO_ARG,       "disables all SUBSTITUTE and APPEND rules"),
