@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2017, GrammarSoft ApS
+* Copyright (C) 2007-2018, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -49,6 +49,7 @@ enum OPTIONS {
 	SUB_DELIMITER,
 	SUB_RTL,
 	SUB_LTR,
+	ORDERED,
 	NUM_OPTIONS,
 };
 
@@ -58,14 +59,14 @@ UOption options[] = {
 	UOPTION_DEF_D("prefix",       'p', UOPT_REQUIRES_ARG, "sets the mapping prefix; defaults to @"),
 	UOPTION_DEF_D("in-auto",      'u', UOPT_NO_ARG,       "auto-detect input format (default)"),
 	UOPTION_DEF_D("in-cg",        'c', UOPT_NO_ARG,       "sets input format to CG"),
-	UOPTION_DEF_D("v",            'v', UOPT_NO_ARG,       "!"),
+	UOPTION_DEF_D("v",            'v', UOPT_NO_ARG,       nullptr),
 	UOPTION_DEF_D("in-niceline",  'n', UOPT_NO_ARG,       "sets input format to Niceline CG"),
 	UOPTION_DEF_D("in-apertium",  'a', UOPT_NO_ARG,       "sets input format to Apertium"),
 	UOPTION_DEF_D("in-fst",       'f', UOPT_NO_ARG,       "sets input format to HFST/XFST"),
 	UOPTION_DEF_D("in-plain",     'p', UOPT_NO_ARG,       "sets input format to plain text"),
 	UOPTION_DEF_D("add-tags",       0, UOPT_NO_ARG,       "adds minimal analysis to readings (implies -p)"),
 	UOPTION_DEF_D("out-cg",       'C', UOPT_NO_ARG,       "sets output format to CG (default)"),
-	UOPTION_DEF_D("V",            'V', UOPT_NO_ARG,       "!"),
+	UOPTION_DEF_D("V",            'V', UOPT_NO_ARG,       nullptr),
 	UOPTION_DEF_D("out-apertium", 'A', UOPT_NO_ARG,       "sets output format to Apertium"),
 	UOPTION_DEF_D("out-matxin",   'M', UOPT_NO_ARG,       "sets output format to Matxin"),
 	UOPTION_DEF_D("out-niceline", 'N', UOPT_NO_ARG,       "sets output format to Niceline CG"),
@@ -75,6 +76,7 @@ UOption options[] = {
 	UOPTION_DEF_D("sub-delim",    'S', UOPT_REQUIRES_ARG, "FST sub-reading delimiters (defaults to #)"),
 	UOPTION_DEF_D("rtl",          'r', UOPT_NO_ARG,       "sets sub-reading direction to RTL (default)"),
 	UOPTION_DEF_D("ltr",          'l', UOPT_NO_ARG,       "sets sub-reading direction to LTR"),
+	UOPTION_DEF_D("ordered",      'o', UOPT_NO_ARG,       "tag order matters mode"),
 };
 }
 
