@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2017, GrammarSoft ApS
+* Copyright (C) 2007-2018, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -31,7 +31,7 @@
 namespace CG3 {
 
 template<typename State>
-Tag *parseTag(const UChar *to, const UChar *p, State& state) {
+Tag* parseTag(const UChar* to, const UChar* p, State& state) {
 	if (to[0] == 0) {
 		state.error("%s: Error: Empty tag on line %u near `%S`! Forgot to fill in a ()?\n", p);
 	}
@@ -49,11 +49,11 @@ Tag *parseTag(const UChar *to, const UChar *p, State& state) {
 		return it->second;
 	}
 
-	Tag *tag = state.get_grammar()->allocateTag();
+	Tag* tag = state.get_grammar()->allocateTag();
 	tag->type = 0;
 
 	if (to[0]) {
-		const UChar *tmp = to;
+		const UChar* tmp = to;
 		while (tmp[0] && (tmp[0] == '!' || tmp[0] == '^')) {
 			if (tmp[0] == '!' || tmp[0] == '^') {
 				tag->type |= T_FAILFAST;
@@ -69,7 +69,6 @@ Tag *parseTag(const UChar *to, const UChar *p, State& state) {
 			u_fflush(state.ux_stderr);
 		}
 
-		// ToDo: Implement META and VAR
 		if (tmp[0] == 'M' && tmp[1] == 'E' && tmp[2] == 'T' && tmp[3] == 'A' && tmp[4] == ':') {
 			tag->type |= T_META;
 			tmp += 5;
@@ -166,7 +165,7 @@ Tag *parseTag(const UChar *to, const UChar *p, State& state) {
 		// ToDo: Remove for real ordered mode
 		if (tag->type & T_REGEXP_LINE) {
 			constexpr UChar uu[] = { '_', '_', 0 };
-			constexpr UChar rx[] = { '(', '^', '|', '$', '|', ' ', '|', ' ', '.', '+', '?', ' ', ')', 0 }; // (^|$| | .+? )
+			constexpr UChar rx[] = { '(', '?', ':', '^', '|', '$', '|', ' ', '|', ' ', '.', '+', '?', ' ', ')', 0 }; // (^|$| | .+? )
 			size_t pos;
 			while ((pos = tag->tag.find(uu)) != UString::npos) {
 				tag->tag.replace(pos, 2, rx);
@@ -176,7 +175,7 @@ Tag *parseTag(const UChar *to, const UChar *p, State& state) {
 
 		for (auto iter : state.get_grammar()->regex_tags) {
 			UErrorCode status = U_ZERO_ERROR;
-			uregex_setText(iter, tag->tag.c_str(), tag->tag.size(), &status);
+			uregex_setText(iter, tag->tag.c_str(), static_cast<int32_t>(tag->tag.size()), &status);
 			if (status != U_ZERO_ERROR) {
 				state.error("%s: Error: uregex_setText(parseTag) returned %s on line %u near `%S` - cannot continue!\n", u_errorName(status), p);
 			}
@@ -187,7 +186,7 @@ Tag *parseTag(const UChar *to, const UChar *p, State& state) {
 		}
 		for (auto iter : state.get_grammar()->icase_tags) {
 			UErrorCode status = U_ZERO_ERROR;
-			if (u_strCaseCompare(tag->tag.c_str(), tag->tag.size(), iter->tag.c_str(), iter->tag.size(), U_FOLD_CASE_DEFAULT, &status) == 0) {
+			if (u_strCaseCompare(tag->tag.c_str(), static_cast<int32_t>(tag->tag.size()), iter->tag.c_str(), static_cast<int32_t>(iter->tag.size()), U_FOLD_CASE_DEFAULT, &status) == 0) {
 				tag->type |= T_TEXTUAL;
 			}
 			if (status != U_ZERO_ERROR) {
@@ -260,10 +259,10 @@ Tag *parseTag(const UChar *to, const UChar *p, State& state) {
 				}
 
 				if (tag->type & T_CASE_INSENSITIVE) {
-					tag->regexp = uregex_open(rt.c_str(), rt.length(), UREGEX_CASE_INSENSITIVE, &pe, &status);
+					tag->regexp = uregex_open(rt.c_str(), static_cast<int32_t>(rt.size()), UREGEX_CASE_INSENSITIVE, &pe, &status);
 				}
 				else {
-					tag->regexp = uregex_open(rt.c_str(), rt.length(), 0, &pe, &status);
+					tag->regexp = uregex_open(rt.c_str(), static_cast<int32_t>(rt.size()), 0, &pe, &status);
 				}
 				if (status != U_ZERO_ERROR) {
 					state.error("%s: Error: uregex_open returned %s trying to parse tag %S on line %u near `%S` - cannot continue!\n", u_errorName(status), tag->tag.c_str(), p);
@@ -293,7 +292,7 @@ Tag *parseTag(const UChar *to, const UChar *p, State& state) {
 }
 
 template<typename State>
-Set *parseSet(const UChar *name, const UChar *p, State& state) {
+Set* parseSet(const UChar* name, const UChar* p, State& state) {
 	uint32_t sh = hash_value(name);
 
 	if (ux_isSetOp(name) != S_IGNORE) {
@@ -303,15 +302,15 @@ Set *parseSet(const UChar *name, const UChar *p, State& state) {
 	if ((
 	      (name[0] == '$' && name[1] == '$') || (name[0] == '&' && name[1] == '&')) &&
 	    name[2]) {
-		const UChar *wname = &(name[2]);
+		const UChar* wname = &(name[2]);
 		uint32_t wrap = hash_value(wname);
-		Set *wtmp = state.get_grammar()->getSet(wrap);
+		Set* wtmp = state.get_grammar()->getSet(wrap);
 		if (!wtmp) {
 			state.error("%s: Error: Attempted to reference undefined set '%S' on line %u near `%S`!\n", wname, p);
 		}
-		Set *tmp = state.get_grammar()->getSet(sh);
+		Set* tmp = state.get_grammar()->getSet(sh);
 		if (!tmp) {
-			Set *ns = state.get_grammar()->allocateSet();
+			Set* ns = state.get_grammar()->allocateSet();
 			ns->line = state.get_grammar()->lines;
 			ns->setName(name);
 			ns->sets.push_back(wtmp->hash);
@@ -327,12 +326,12 @@ Set *parseSet(const UChar *name, const UChar *p, State& state) {
 	if (state.get_grammar()->set_alias.find(sh) != state.get_grammar()->set_alias.end()) {
 		sh = state.get_grammar()->set_alias[sh];
 	}
-	Set *tmp = state.get_grammar()->getSet(sh);
+	Set* tmp = state.get_grammar()->getSet(sh);
 	if (!tmp) {
 		if (!state.strict_tags.empty() || !state.list_tags.empty()) {
-			Tag *tag = parseTag(name, p, state);
+			Tag* tag = parseTag(name, p, state);
 			if (state.strict_tags.count(tag->plain_hash) || state.list_tags.count(tag->plain_hash)) {
-				Set *ns = state.get_grammar()->allocateSet();
+				Set* ns = state.get_grammar()->allocateSet();
 				ns->line = state.get_grammar()->lines;
 				ns->setName(name);
 				state.get_grammar()->addTagToSet(tag, ns);
